# Copyright (C) 2008, Urko Fernandez <tturktime@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import logging
import hippo
import gtk
import questionnaire
import os
import difflib
import hint

from toolbars import *
from sugar.graphics.toolbutton import ToolButton
from sugar.graphics.toggletoolbutton import ToggleToolButton

from sugar.graphics.roundbox import CanvasRoundBox
from sugar.graphics.style import (Color, COLOR_BLACK, COLOR_WHITE, 
    FONT_BOLD, FONT_NORMAL)
from sugar.activity import activity
from gettext import gettext as _
from datetime import datetime
from sugar.graphics import style
from cardpanel import CardPanel
from controlpanel import ControlPanel

logger = logging.getLogger('assimilate-activity')


#TODO: change copy paste from that of read activity, maybe in sugar.clipboard

class Assimilate(activity.Activity):



    #It cannot be 0 because it would assert as false as in None type
    _EASYLY_ANSWERED = 1
    _HARDLY_ANSWERED = 2
    _FORGOTTEN_ANSWER = 3 #But answer was known
    _DUNNO_ANSWER = 4
    _DEFAULT_COLOR_STROKE = '#000000'
    _DEFAULT_COLOR_FILL = '#888888'
    
    def __init__(self, handle):
#The __init__() method is also used to make the distinction between being resumed from 
#the Journal, or starting with a blank document.

#TODO:Implement read_file() and write_file() (for the journal)
        activity.Activity.__init__(self, handle)       
        logger.debug('Starting Assimilate activity...')
        self.buddy = self._pservice.get_owner()
        
        self.card_panel = CardPanel(self)
        
        self.control_panel = ControlPanel(self)

        vbox = hippo.CanvasBox(spacing=4,
            orientation=hippo.ORIENTATION_VERTICAL, background_color=style.Color('#444444').get_int()) 
#TODO: 0x444444ff is '#444444' this color looks good, but needs changes in some widgets borders

        hbox = hippo.CanvasBox(spacing=4,
            orientation=hippo.ORIENTATION_HORIZONTAL)


        hbox.append(self.card_panel, hippo.PACK_EXPAND)
        hbox.append(hippo.CanvasWidget(widget=gtk.VSeparator()))
        hbox.append(self.control_panel, hippo.PACK_END)


                
        vbox.append(hbox, hippo.PACK_EXPAND)
        
        canvas = hippo.Canvas()
        canvas.set_root(vbox)
        self.set_canvas(canvas)
        self.show_all()

        toolbox = activity.ActivityToolbox(self)
        self.set_toolbox(toolbox)
        self.toolbox.show()

        self.toolbox.add_toolbar(_("Edit"), AssimilationEditToolbar(self))
        self.toolbox.add_toolbar(_("Quiz Mode"), QuizModeToolbar(self))
        self.toolbox.add_toolbar(_("Modify"), ModifyToolbar(self))
        self.toolbox.add_toolbar(_("questionnaire"), questionnaireToolbar(self))
        self.toolbox.add_toolbar(_("Help"), HelpToolbar(self))
        self.toolbox.show_all()
        self.set_toolbox(toolbox)


        nick = self.buddy.props.nick

        self.color_stroke , self.color_fill = self.buddy.props.color.split(',')
        
        self.get_questionaries()
        for questionnaire_name in self.questionnaire_dict.values():
            self.current_questionnaire = questionnaire_name
            break
        self.initialize_questionnaire()
        
    def initialize_questionnaire(self):
        self.current_questionnaire.read_flashcards()
        self.current_questionnaire.add_buddy(self.buddy)
        logging.info('Just before loadflash')
        self.load_flashcard()
        #If the user doesn't evaluate his answering, no result is to be recorded when flipping cards
        self.answering_mode = None
        
    def load_flashcard(self):
        self.current_card_time = datetime.today()
        #Each time user sees an answer, karma goes up and in case he rephrases the answer it probably be a good one
        self.answer_karma = 0
        self.question_karma = 0
           
        if self.current_questionnaire.current_flashcard:
            self.current_questionnaire.read_questions()
            if len(self.current_questionnaire.question_dict) > 1:
                self.card_panel.rephrase_question_button.set_sensitive(True)
            else:
                self.card_panel.rephrase_question_button.set_sensitive(False)
            self.current_questionnaire.read_answers()
                        
            if self.current_questionnaire.current_answer:
                self.card_panel.get_answer_button.set_sensitive(True)
                self.card_panel.check_answer_button.set_sensitive(True)
                #answer hint object contains current flashcards answers and can show them in different ways
                self.answer_hint = hint.hint(self, self.current_questionnaire.answer_dict, 
                  self.card_panel.answer_rb)
                logging.info('A.load_flashcard answer_dict = %s' % self.current_questionnaire.answer_dict)
            else:
                self.card_panel.get_answer_button.set_sensitive(False)
                self.card_panel.check_answer_button.set_sensitive(False)

            #Set hscale slider's new adjustment
            #logging.info('A.load_flashcard, flashcard_dict is %s' % self.current_questionnaire.flashcard_dict)
            #This is a Dict containing the id and value of each question in the opened questionnaire
            self.control_panel.scale_adjustment = gtk.Adjustment(0.0, 0.0, len(self.current_questionnaire.flashcard_dict) -1, 1, 0.0, 0.0)
            self.control_panel.flashcard_slider.set_adjustment(self.control_panel.scale_adjustment)
            self.control_panel.scale_adjustment.connect("changed", self._on_slider_changed)
            #there are two similar callbacks, "changed" and "value_changed"
#TODO: There are many checks done throughout the code using a simple conditional check, decide wether exception handling is better or not        
            logging.info('A.load_flashcard, current_question=%s' % (self.current_questionnaire.current_question))            
            if self.current_questionnaire.current_question:
                buddyid = self.current_questionnaire.get_question_owner(self.current_questionnaire.current_question)
                logging.info('A.load_flashcard,, questions buddyid=%s' % (buddyid))
                color_stroke, color_fill = self.current_questionnaire.get_buddy_colors(buddyid)
                self.card_panel.write_question(self.current_questionnaire.question_dict[self.current_questionnaire.first_question], color_stroke, color_fill)
            else:
                self.card_panel.write_question(_('Write a question with the modify menu'), self._DEFAULT_COLOR_STROKE, self._DEFAULT_COLOR_FILL)
                #logging.info('there are no questions to write, you should create one!')
                self.card_panel.answer_entry.set_editable(False)
                self.card_panel.tag_entry.set_editable(False)
            
            #Populating subject combobox needs to be after creating a questionnaire object
            self.write_subject_list()
            
            self.card_panel.write_answer('', self._DEFAULT_COLOR_STROKE, self._DEFAULT_COLOR_FILL)
            self.card_panel.answer_entry.set_text('')
            
            self.card_panel.tag_rb.remove_all()
            self.write_tag_list()
            #logging.info('A.load_flashcard, current_answer = %s' % self.current_questionnaire.current_answer)
        else:
            #Add a new empty flashcard to the database and show it
            deck = 1
            self.current_questionnaire.add_flashcard(deck)
#TODO: is it necessary to do this recursion? Test what happens when deleting last card   
            self.load_flashcard()
        
           
    def get_questionaries(self):
        #Don't know if I should attach this method to this instance, activity is always one, maybe 
        #a simple function should be enough.
        self.questionnaire_dict = {}
        for db in os.listdir('data'):
            if db.split('.')[1] == 'sqlite':
                self.questionnaire_dict[db.split('.')[0]] = questionnaire.questionnaire('data/'+db)
        if not self.questionnaire_dict:
            #There are no databases, create a new one
            logging.info('A.get_questionaries, about to create default questionnaire')
            self.questionnaire_dict['default'] = questionnaire.questionnaire('data/default.sqlite')
    
    def change_questionnaire(self, questionnaire_name):
        logging.info('A.change_questionnaire, questionar_name = %s, q_dict=%s' % (questionnaire_name, self.questionnaire_dict))
        
        if questionnaire_name:
            self.current_questionnaire = self.questionnaire_dict[questionnaire_name]
        else:
            self.questionnaire_dict['default'] = questionnaire.questionnaire('data/default.sqlite')
        self.current_answer = None
        self.current_question = None
        self.current_flashcard = None
        self.current_subject = None
        
        self.initialize_questionnaire()       
    
    def _on_slider_changed(self, adj):
#TODO: Needs some work to relate id and represent value in the slide  
        #logging.info('A._on_slider_changed, lenght of flashcard_dict is %s' % len(self.current_questionnaire.flashcard_dict))
        self.current_questionnaire.current_flashcard = int(adj.value)
        self.load_flashcard()

    def write_subject_list(self):
        self.current_questionnaire.read_subjects()
        subjectid = self.current_questionnaire.get_subject_from_card(self.current_questionnaire.current_flashcard)
        if subjectid != "NULL":
            self.current_questionnaire.current_subject = subjectid
        else:
            self.current_questionnaire.change_subject('No subject')
            self.current_questionnaire.read_subjects()
            subjectid = self.current_questionnaire.get_subject_from_card(self.current_questionnaire.current_flashcard)
        self.create_subject_combobox(self.control_panel.subject_combobox, self.control_panel.subject_combo_changed_id)

    def create_subject_combobox(self, combobox, combo_changed_id):
        combobox.handler_block(combo_changed_id)
        combobox.remove_all()
        combobox.handler_unblock(combo_changed_id)
       
#TODO:Need to insert action_id, don't know what it is. The following is an ugly temporal solution
        i = 0
        for subjectid in self.current_questionnaire.subject_dict:
            combobox.append_item(subjectid, self.current_questionnaire.subject_dict[subjectid])
        #Can access subject name having subjectid via current_questionnaire.subject_dict or using curren_questionnaire.get_subject method
            if (self.current_questionnaire.current_subject is not None and 
                self.current_questionnaire.get_subject_name(self.current_questionnaire.current_subject) == 
                    self.current_questionnaire.subject_dict[subjectid]):
                #Block this widget's handlers to avoid entering in an infinite loop
                combobox.handler_block(combo_changed_id)
                combobox.set_active(i)
                combobox.handler_unblock(combo_changed_id)
            i += 1
        #logging.info('A.createSubjectCombobox, setting active subject id=%s' % i)
        if combobox.get_active() is None:
            #Run the else if it didn't exit with a break.
            combobox.handler_block(combo_changed_id)
            combobox.set_active(-1)
            combobox.handler_unblock(combo_changed_id)
          
    def write_tag_list(self):
#TODO: logarithmically change font size
        self.current_questionnaire.read_tags()
        tag_count = {}
        for tag in self.current_questionnaire.tag_dict.keys():
            number_of_tags = self.current_questionnaire.get_number_of_tags(tag)
            tag_count[self.current_questionnaire.tag_dict[tag]] = number_of_tags

        
        tag_font = style.FONT_NORMAL.get_pango_desc()
        self.card_panel.tag_rb.remove_all()
        for tag in tag_count.keys():
            tag_font.set_size(10000 + 5000 * (tag_count[tag]))
            #logging.info('A.write_tag_list, tag font size %s' % tag_font.get_size())
            self.card_panel.write_tag(_(tag + '  '), tag_font)
    
    def record_result(self):   
        if self.answering_mode:
            #logging.info('A.record_result question=%s answer=%s' % (self.current_questionnaire.current_question, self.current_questionnaire.current_answer))
            self.current_questionnaire.add_result(0, self.answering_mode, 
                                               datetime.today(),
                                               self.current_questionnaire.current_flashcard)
            if self.answering_mode == self._EASYLY_ANSWERED:
                self.control_panel.no_problem_button.set_active(False)
                self.answering_mode = None
            elif self.answering_mode == self._HARDLY_ANSWERED:
                self.control_panel.problem_button.set_active(False)
                self.answering_mode = None
            elif self.answering_mode == self._FORGOTTEN_ANSWER:
                self.control_panel.forgotten_button.set_active(False)
                self.answering_mode = None
            else:
                #self.answering_mode == self._DUNNO_ANSWER:
                self.control_panel.dunno_button.set_active(False)
                self.answering_mode = None
    @staticmethod            
    def compare_sequences(seq1, seq2):
        """compare two strings and return similarity percentage"""
        for punctuation_mark in '!?.,:;-+':
            seq1 = seq1.replace(punctuation_mark, ' ')
            seq2 = seq2.replace(punctuation_mark, ' ')
            #All lowercase, punctuation marks replaced with space symbol and converted to a list in order to sort it
        list1 = seq1.lower().split()
        list2 = seq2.lower().split()
        
        list1.sort()
        list2.sort()
        ##logging.info('A.compare_sequences list1=%s and list2=%s' % (list1, list2))
        seq1 = seq2 = ''
        for i in list1:
            seq1 = seq1 + i + ' '
        for i in list2:
            seq2 = seq2 + i + ' '
        #strip spaces from the end
        seq1 = seq1.strip(' ')
        seq2 = seq2.strip(' ')
        sm = difflib.SequenceMatcher(None, seq1, seq2)
        return  round(sm.ratio(),3)
    
    def find_similar_text(self, text, text_dict):
        '''finds similar text in a given dictionary with texts as values'''
        
        #This same method is used with subjects, tags, answers and questions
        identical_text = None
        similar_text = None
        identical_ratio = 0
        similar_ratio = 0
        logging.info('A.find_similar_text, text=%s,text_dict=%s' % (text,text_dict))
        if text:
            for existing_text in text_dict.values():
                ratio = self.compare_sequences(text, existing_text)           
                if ratio >= 0.95 and (not identical_text or ratio > identical_ratio):
                    identical_text = existing_text
                    identical_ratio = ratio
                elif ratio >= 0.75 and (not similar_text or ratio > similar_ratio):
                    similar_text = existing_text
                    similar_ratio = ratio
            logging.info('A.find_similar_text, identical_ratio=%s, similar_ratio=%s' % (identical_ratio,similar_ratio))
            
            if identical_text:
                return (identical_ratio, identical_text)
            elif similar_text:
                return (similar_ratio, similar_text)
            else:
                return (None, text)
    
    def add_answer_to_current_card(self, answer):
        #logging.info('A.add_answer_to_current_card, karma is %s'% karma)
        self.current_questionnaire.add_answer(self.answer_karma, answer, self.current_questionnaire.current_flashcard, 
            self.current_questionnaire.get_buddyid(self.buddy.props.nick))
        self.answer_karma = 0
        self.card_panel.write_answer(answer, self.color_stroke , self.color_fill)
        #TODO: we created a new object maybe over an existing one losing the state we were in
        #    update or create a new one if none exists
        #self.answer_hint = hint.hint(self, self.current_questionnaire.answer_dict, 
         #         self.card_panel.answer_rb)
        
    def add_question_to_current_card(self, question):
        #logging.info('A.add_question_to_current_card buddy is %s' % self.buddy.props.nick)
        self.current_questionnaire.add_question(self.question_karma, question, self.current_questionnaire.current_flashcard,
                    self.current_questionnaire.get_buddyid(self.buddy.props.nick))
        self.card_panel.write_question(question, self.color_stroke , self.color_fill)
    
    def add_subject_to_current_card(self, subject):
        self.current_questionnaire.change_subject(subject)
#TODO: how do you find a value in a combobox and get it's index number without iterating and comparing?
        #logging.info('on A._alert_subject_cb, current_subject = %s' % self.current_questionnaire.current_subject)
        self.write_subject_list()
    
    def add_tag_to_current_card(self, tag):
        tagid = self.current_questionnaire.add_tag(self.current_questionnaire.current_flashcard, tag)
        buddyid = self.current_questionnaire.get_buddyid(self.buddy.props.nick)
        #logging.info('A.add_tag_to_current_card, tagid=%s buddyid=%s' %(tagid, buddyid))
        self.current_questionnaire.add_buddy_tag(tagid, buddyid)
        self.write_tag_list()
    
    def disable_widgets(self):
        self.card_panel.disable_widgets()
        self.control_panel.disable_widgets()
        #self.toolbars.disable_toolbar()
        #TODO: add .props.sensitive = False in each toolbutton
        
    def enable_widgets(self):
        self.card_panel.enable_widgets()
        self.control_panel.enable_widgets()
        #self.toolbars.enable_toolbar()
      
    def _alert_question_cb(self, alert, response_id, question):
        if response_id is gtk.RESPONSE_OK:
            #logging.debug('T._alert_question_cb Clicked Continue')
            self.add_question_to_current_card(question)
        self.remove_alert(alert) 
        
    def _alert_subject_cb(self, alert, response_id, subject):        
        if response_id is gtk.RESPONSE_OK:
            self.add_subject_to_current_card(subject)
        self.remove_alert(alert) 
        

