import gtk
import hippo
import logging
import toolbars
import pango

from gettext import gettext as _
from sugar.graphics import style
from sugar.graphics.roundbox import CanvasRoundBox
from sugar.graphics.alert import ConfirmationAlert, NotifyAlert
from datetime import datetime


logger = logging.getLogger('assimilate-activity.cardpanel')

class CardPanel(hippo.CanvasBox):
    
    def __init__(self, handler):
        
        self._handler = handler
        self._font = style.FONT_NORMAL.get_pango_desc()
        self._font.set_size(20000)
        
        hippo.CanvasBox.__init__(self, spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL)

        self.header_font = style.FONT_BOLD.get_pango_desc()
        self.header_font.set_size(16000)
        
        self.create_question_box()
        self.create_answer_box()
        self.create_test_box()
        self.create_tag_box()
        
        self.text_color = style.COLOR_WHITE.get_int() 
    
    def create_question_box(self):    
        self.question_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL)

        
        question_header = hippo.CanvasText(text='Question', color=style.COLOR_WHITE.get_int(),
                          font_desc= self.header_font)
        self.question_box.append(question_header)
        self.append(self.question_box, hippo.PACK_CLEAR_LEFT)
        #self.append(hippo.CanvasWidget(widget=gtk.HSeparator()))
        
        #Canvas elements have box_width and box_height attributes among others
        self.question_rb = CanvasRoundBox(background_color=style.Color(self._handler._DEFAULT_COLOR_FILL).get_int(),
                            border_color=style.Color(self._handler._DEFAULT_COLOR_STROKE).get_int(),
                            padding=4, box_height=160,
                            xalign=hippo.ALIGNMENT_CENTER,
                            yalign=hippo.ALIGNMENT_CENTER)
        #self.question_rb.props.border_color = style.Color(self._handler._DEFAULT_COLOR_STROKE).get_int()
        
        self.question_box.append(self.question_rb)
                
        self.rephrase_question_button = gtk.Button()
        rephrase_question_label = gtk.Label()
        rephrase_question_label.set_markup(_('<big><b>Rephrase</b></big>'))
        self.rephrase_question_button.add(rephrase_question_label)
        self.rephrase_question_button.connect('clicked', self._on_rephrase_question_button_clicked_cb)
        self.rephrase_question_box = hippo.CanvasBox(xalign=hippo.ALIGNMENT_CENTER)
        self.rephrase_question_box.append(hippo.CanvasWidget(widget=self.rephrase_question_button, box_width=style.GRID_CELL_SIZE * 1.5 , box_height=style.GRID_CELL_SIZE * 0.5))
        self.question_box.append(self.rephrase_question_box)
    
    def create_answer_box(self):    
        self.answer_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL)
        answer_header = hippo.CanvasText(text='Answer', color=style.COLOR_WHITE.get_int(),
                          font_desc= self.header_font)

        self.answer_box.append(answer_header)
        self.append(self.answer_box, hippo.PACK_CLEAR_LEFT)
        #self.append(hippo.CanvasWidget(widget=gtk.HSeparator()))
        
        self.answer_rb = CanvasRoundBox(background_color=style.Color(self._handler._DEFAULT_COLOR_FILL).get_int(),
                            border_color=style.Color(self._handler._DEFAULT_COLOR_STROKE).get_int(),
                            padding=4, box_height=160,
                            xalign=hippo.ALIGNMENT_CENTER,
                            yalign=hippo.ALIGNMENT_CENTER)
        #self.answer_rb.props.border_color = color_stroke
        
        self.answer_box.append(self.answer_rb)
        
        self.get_answer_button = gtk.Button()
        get_answer_label = gtk.Label()
        get_answer_label.set_markup(_('<big><b>Show</b></big>'))
        self.get_answer_button.add(get_answer_label)
        self.get_answer_button.connect('clicked', self._on_get_answer_button_clicked_cb)
        self.get_answer_box = hippo.CanvasBox(xalign=hippo.ALIGNMENT_CENTER)
        self.get_answer_box.append(hippo.CanvasWidget(widget=self.get_answer_button, box_width=style.GRID_CELL_SIZE * 1.5 , box_height=style.GRID_CELL_SIZE * 0.5))
        self.answer_box.append(self.get_answer_box)
        self.answer_rb.set_clickable(True)
        self.answer_rb.connect('activated', self._on_get_answer_button_clicked_cb)
        self.answer_entry = gtk.Entry()
        self.answer_entry.connect('focus-in-event', self._on_focus_on_answer_entry)
        self.answer_entry.connect('activate', self._on_check_answer_button_clicked_cb)
        self.answer_entry_box = hippo.CanvasBox(
                orientation=hippo.ORIENTATION_HORIZONTAL)
        self.answer_entry_box.append(hippo.CanvasWidget(widget=self.answer_entry), hippo.PACK_EXPAND)


        self.check_answer_button = gtk.Button()
   
        check_answer_label = gtk.Label()
        check_answer_label.set_markup(_('<big><b>Check</b></big>'))
        self.check_answer_button.add(check_answer_label)
        self.check_answer_button.connect('clicked', self._on_check_answer_button_clicked_cb)
        
        self.answer_entry_box.append(hippo.CanvasWidget(widget=self.check_answer_button, box_width=style.GRID_CELL_SIZE * 1.5 , box_height=style.GRID_CELL_SIZE * 0.5))
        
        self.answer_box.append(self.answer_entry_box)
        
    def create_tag_box(self):
        self.tag_box = hippo.CanvasBox(
                orientation=hippo.ORIENTATION_VERTICAL)
        tag_header = hippo.CanvasText(text='Tag list', color=style.COLOR_WHITE.get_int(),
                          font_desc= self.header_font)
        self.tag_box.append(tag_header)
        self.append(self.tag_box, hippo.PACK_CLEAR_LEFT)

       
        self.tag_rb = CanvasRoundBox(background_color=style.Color(self._handler._DEFAULT_COLOR_FILL).get_int(),
                            border_color=style.Color(self._handler._DEFAULT_COLOR_STROKE).get_int(),
                            xalign=hippo.ALIGNMENT_CENTER,
                            box_height=40)
        #self.tag_rb.props.border_color = color_stroke
        
        self.tag_box.append(self.tag_rb)
        self.tag_entry = gtk.Entry()
        self.tag_entry.connect('activate', self._on_add_tag_button_clicked_cb, self.tag_entry)
       
        #self.set_properties(yalign=hippo.ALIGNMENT_CENTER,
                                  #box_width=self._PROGRESS_COL_WIDTH)
        #self.tag_box.append(hippo.CanvasWidget(widget=self.tag_entry, xalign=hippo.ALIGNMENT_CENTER,
              #                    box_width=style.GRID_CELL_SIZE * 4))
        
        self.tag_button = gtk.Button()
        tag_button_label = gtk.Label()
        tag_button_label.set_markup('<big><b>Tag it</b></big>')
        self.tag_button.add(tag_button_label)
        self.tag_button.connect('clicked', self._on_add_tag_button_clicked_cb, self.tag_entry)
        self.add_tag_box = hippo.CanvasBox(xalign=hippo.ALIGNMENT_CENTER, orientation=hippo.ORIENTATION_HORIZONTAL)
        self.add_tag_box.append(hippo.CanvasWidget(widget=self.tag_entry))
        self.add_tag_box.append(hippo.CanvasWidget(widget=self.tag_button, box_width=style.GRID_CELL_SIZE , box_height=style.GRID_CELL_SIZE *0.5))
        self.tag_box.append(self.add_tag_box)

    
    def create_test_box(self):
        self.test_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL,
                xalign=hippo.ALIGNMENT_CENTER)
        #test_header = hippo.CanvasText(text='Answer', color=style.COLOR_WHITE.get_int(),
                          #font_desc= self.header_font)
        
        self.append(self.test_box, hippo.PACK_CLEAR_LEFT)

        #self.append(hippo.CanvasWidget(widget=gtk.HSeparator()))
        
        self.choice_a_rb = CanvasRoundBox(background_color=style.Color(self._handler._DEFAULT_COLOR_FILL).get_int(),
                            border_color=style.Color(self._handler._DEFAULT_COLOR_STROKE).get_int(),
                            padding=4,box_width=style.GRID_CELL_SIZE * 6, box_height=style.GRID_CELL_SIZE,
                            xalign=hippo.ALIGNMENT_CENTER,
                            yalign=hippo.ALIGNMENT_CENTER)
        self.choice_a_rb.set_clickable(True)
        self.choice_a_rb.connect('activated', self._on_choice_a_rb_clicked_cb)
        self.choice_b_rb = CanvasRoundBox(background_color=style.Color(self._handler._DEFAULT_COLOR_FILL).get_int(),
                            border_color=style.Color(self._handler._DEFAULT_COLOR_STROKE).get_int(),
                            padding=4, box_width=style.GRID_CELL_SIZE * 6, box_height=style.GRID_CELL_SIZE,
                            xalign=hippo.ALIGNMENT_CENTER,
                            yalign=hippo.ALIGNMENT_CENTER)
        self.choice_b_rb.set_clickable(True)
        self.choice_b_rb.connect('activated', self._on_choice_b_rb_clicked_cb)
        self.choice_c_rb = CanvasRoundBox(background_color=style.Color(self._handler._DEFAULT_COLOR_FILL).get_int(),
                            border_color=style.Color(self._handler._DEFAULT_COLOR_STROKE).get_int(),
                            padding=4, box_width=style.GRID_CELL_SIZE * 6, box_height=style.GRID_CELL_SIZE,
                            xalign=hippo.ALIGNMENT_CENTER,
                            yalign=hippo.ALIGNMENT_CENTER)
        self.choice_c_rb.set_clickable(True)
        self.choice_c_rb.connect('activated', self._on_choice_c_rb_clicked_cb)
        self.choice_d_rb = CanvasRoundBox(background_color=style.Color(self._handler._DEFAULT_COLOR_FILL).get_int(),
                            border_color=style.Color(self._handler._DEFAULT_COLOR_STROKE).get_int(),
                            padding=4, box_width=style.GRID_CELL_SIZE * 6, box_height=style.GRID_CELL_SIZE,
                            xalign=hippo.ALIGNMENT_CENTER,
                            yalign=hippo.ALIGNMENT_CENTER)
        self.choice_d_rb.set_clickable(True)
        self.choice_d_rb.connect('activated', self._on_choice_d_rb_clicked_cb)
        self.test_upper_row_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_HORIZONTAL,
                xalign=hippo.ALIGNMENT_CENTER)
        self.test_middle_row_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_HORIZONTAL,
                xalign=hippo.ALIGNMENT_CENTER)
        self.test_lower_row_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_HORIZONTAL,
                xalign=hippo.ALIGNMENT_CENTER)
        self.test_upper_row_box.append(self.choice_a_rb)
        self.test_middle_row_box.append(self.choice_b_rb)
        self.test_middle_row_box.append(self.choice_c_rb)
        self.test_lower_row_box.append(self.choice_d_rb)
        self.test_box.append(self.test_upper_row_box)
        self.test_box.append(self.test_middle_row_box)
        self.test_box.append(self.test_lower_row_box)
        
        self.show_choice_button = gtk.Button()
        show_choice_label = gtk.Label()
        show_choice_label.set_markup(_('<big><b>Show</b></big>'))
        self.show_choice_button.add(show_choice_label)
        self.show_choice_button.connect('clicked', self._on_show_choice_button_clicked_cb)
        
        self.show_choice_box = hippo.CanvasBox(xalign=hippo.ALIGNMENT_CENTER)
        self.show_choice_box.append(hippo.CanvasWidget(widget=self.show_choice_button, box_width=style.GRID_CELL_SIZE * 1.5 , box_height=style.GRID_CELL_SIZE * 0.5))
        self.test_box.append(self.show_choice_box)
        
        self.test_box.set_visible(False)
        self.show_choice_box.set_visible(False)

       
    def write_question(self, question, color_stroke, color_fill):
        self.question_rb.props.border_color = style.Color(color_stroke).get_int()
        self.question_rb.props.background_color = style.Color(color_fill).get_int()
        self.question_rb.remove_all()
        if question is not None: 
             self.question_text = hippo.CanvasText(text=question,
                size_mode=hippo.CANVAS_SIZE_WRAP_WORD,
                color=self.text_color,
                font_desc=self._font)  
             self.answer_entry.set_editable(True)
             self.tag_entry.set_editable(True)
        else:
            self.question_text = hippo.CanvasText(text=_('No questions for this flashcard')
        ,
                                size_mode=hippo.CANVAS_SIZE_WRAP_WORD,
                color=self.text_color,
                font_desc=self._font)
            self.answer_entry.set_editable(False)
            self.tag_entry.set_editable(False)
        self.question_rb.append(self.question_text)
        if question != '':
            if len(self._handler.current_questionnaire.question_dict) > 1:
                self.rephrase_question_button.set_sensitive(True)
            else:
                self.rephrase_question_button.set_sensitive(False)
            self._handler.question_karma += 1                           
        
    def write_answer(self, answer, color_stroke, color_fill):
        self.answer_rb.props.border_color = style.Color(color_stroke).get_int()
        self.answer_rb.props.background_color = style.Color(color_fill).get_int()
        self.answer_rb.remove_all()
        if answer is not None: 
             self.answer_text = hippo.CanvasText(text=answer,
                size_mode=hippo.CANVAS_SIZE_WRAP_WORD,
                color=self.text_color,
                font_desc=self._font)
             logging.info('card.write_answer karma=%s' % self._handler.answer_karma)
        else:
            self.answer_text = hippo.CanvasText(text='No answers for this flashcard'
        ,
                                size_mode=hippo.CANVAS_SIZE_WRAP_WORD,
                color=self.text_color,
                font_desc=self._font)
        self.answer_rb.append(self.answer_text)
        if answer != '':
            if len(self._handler.current_questionnaire.answer_dict) > 1:
                self.get_answer_button.set_sensitive(True)
            else:
                self.get_answer_button.set_sensitive(False)  
            #If user reads answers, the answer she is going to write has more karma points
            if self._handler.answer_karma <= len(self._handler.current_questionnaire.answer_dict):
                self._handler.answer_karma += 1
        
    
    
    def write_tag(self, tag, font):
        tag_text = hippo.CanvasText(text=tag,
                                size_mode=hippo.CANVAS_SIZE_WRAP_WORD,
                color=self.text_color,
                font_desc=font)
        tag_text.set_clickable(True)
        tag_text.connect('activated', self._on_tag_clicked_cb)
        self.tag_rb.append(tag_text)


    def disable_widgets(self):
        self.answer_entry.set_editable(False)
        self.tag_entry.set_editable(False)
        self.get_answer_button.set_sensitive(False)
        self.rephrase_question_button.set_sensitive(False)
        #self.tag_button.set_sensitive(False)
        self.tag_button.props.sensitive= False
    
    def enable_widgets(self):
#TODO: Check if all buttons are to be enabled according to contents in the current card
        self.answer_entry.set_editable(True)
        self.tag_entry.set_editable(True)
        self.get_answer_button.set_sensitive(True)
        self.rephrase_question_button.set_sensitive(True)
        self.tag_button.set_sensitive(True)
                 
    def _on_get_answer_button_clicked_cb(self, widget, data=None):
        self._handler.current_questionnaire.read_answers()
        
        if self.answer_text.props.text:
            #logging.info('card._on_get_answer_button_clicked_cb answer text=%s' % self.answer_text.props.text)
            answer = self._handler.current_questionnaire.next_answer()
            buddyid = self._handler.current_questionnaire.get_answer_owner(self._handler.current_questionnaire.current_answer)
            color_stroke, color_fill = self._handler.current_questionnaire.get_buddy_colors(buddyid)
            self.write_answer(answer, color_stroke, color_fill)
        else:
            buddyid = self._handler.current_questionnaire.get_answer_owner(self._handler.current_questionnaire.current_answer)
            color_stroke, color_fill = self._handler.current_questionnaire.get_buddy_colors(buddyid)
            answer = self._handler.current_questionnaire.answer_dict[self._handler.current_questionnaire.first_rowid('answer', self._handler.current_questionnaire.current_flashcard)]
            self.write_answer(answer, color_stroke, color_fill)
        self._handler.toolbox.set_current_toolbar(toolbars._TOOLBAR_MODIFY)
#TODO: This should read karma and show answers in order
      
    def _on_rephrase_question_button_clicked_cb(self, widget, data=None):
        question = self._handler.current_questionnaire.next_question()
        buddyid = self._handler.current_questionnaire.get_question_owner(self._handler.current_questionnaire.current_question)
        color_stroke, color_fill = self._handler.current_questionnaire.get_buddy_colors(buddyid)
        #logging.info('card._on_rephrase_question_button_clicked_cb, buddyid: %s' % buddyid)
        self.write_question(question, color_stroke, color_fill)
        self._handler.toolbox.set_current_toolbar(toolbars._TOOLBAR_MODIFY)
    
    def _on_add_tag_button_clicked_cb(self, widget, entry): 
        #The entry button is on this method's scope
        if entry.get_text():
            tag = entry.get_text()
            buddyid = self._handler.current_questionnaire.get_buddyid(self._handler.buddy.props.nick)
            buddy_tag_list = self._handler.current_questionnaire.read_user_tags(buddyid)
            logging.info('card._on_add_tag_button_clicked_cb tag=%s and buddy\'s taglist=%s' % (tag, buddy_tag_list))
            if tag not in buddy_tag_list.values():

                (ratio, similar_tag) = self._handler.find_similar_text(tag, self._handler.current_questionnaire.tag_dict)
                logging.info('card._on_add_tag_button_clicked_cb ratio=%s' % ratio)
                if ratio >= 0.95:
                    tagid = self._handler.current_questionnaire.get_tagid(similar_tag)
                    self._handler.current_questionnaire.add_buddy_tag(tagid, buddyid)
                    self._handler.current_questionnaire.related_cards()
                    self._handler.write_tag_list()
                elif ratio >= 0.75:
                    alert = ConfirmationAlert()
                    alert.props.title = "There is a similar tag in this card:"
                    alert.props.msg = '"' + similar_tag + '"'
                    self._handler.add_alert(alert)
                    alert.connect('response', self._alert_tag_cb, tag)
                    alert.show()
                else:
                    self._handler.add_tag_to_current_card(tag)
        
            entry.set_text('')

    def _on_focus_on_answer_entry(self, widget, data=None):
        self._handler.toolbox.set_current_toolbar(toolbars._TOOLBAR_MODIFY)

    def _on_check_answer_button_clicked_cb(self, widget, data=None):
        if self.answer_entry.get_text():
            answer = self.answer_entry.get_text()
            logging.info('card._on_check_answer_button_clicked_cb answer_dict=%s' % self._handler.current_questionnaire.answer_dict)
            (ratio, similar_answer) = self._handler.find_similar_text(answer, self._handler.current_questionnaire.answer_dict)
            alert = NotifyAlert(timeout=5)
            if ratio >= 0.95:
                elapsed_time = datetime.today() - self._handler.current_card_time
                self._handler.current_card_time = datetime.today()
                alert.props.title = "Answer is correct, elapsed time: " + str(elapsed_time.seconds) +" seconds"
                alert.props.msg = '"' + similar_answer + '"'
            elif ratio > 0.75:
                alert.props.title = "There is a similar answer:"
                alert.props.msg = '"' + similar_answer + '"'
            else:
                alert.props.title = "Answer seems to be wrong:"
                alert.props.msg = '"' + similar_answer + '"'
            self._handler.add_alert(alert)
            alert.connect('response', self._alert_check_answer_cb, answer)
            alert.show() 
            self.answer_entry.set_text('')
            
    def _on_tag_clicked_cb(self, widget, data=None):
        tag = widget.props.text
        logging.info('clicked and data=%s and length=%s' % (tag, len(tag)))
        #tags are now written with two trailing spaces
        self._handler.add_tag_to_current_card(tag.strip(' '))
    
    def _on_choice_a_rb_clicked_cb(self, widget, data=None):
        pass
    
    def _on_choice_b_rb_clicked_cb(self, widget, data=None):
        pass
    
    def _on_choice_c_rb_clicked_cb(self, widget, data=None):
        pass
    
    def _on_choice_d_rb_clicked_cb(self, widget, data=None):
        pass
    
    def _on_show_choice_button_clicked_cb(self, widget, data=None):
        pass
    
    def _alert_tag_cb(self, alert, response_id, tag):
        if response_id is gtk.RESPONSE_OK:
            self._handler.add_tag_to_current_card(tag)
        self._handler.remove_alert(alert)
    
    def _alert_check_answer_cb(self, alert, response_id, tag):
        if response_id is gtk.RESPONSE_OK:
            pass
#TODO: Ask if user wants to add answer, guess easily answered if quickly and hardly if tardy
        self._handler.remove_alert(alert)