import gtk
import hippo
import pango
import logging

from gettext import gettext as _
from sugar.graphics.combobox import ComboBox
from sugar.graphics.icon import CanvasIcon
from sugar.graphics.xocolor import XoColor
from sugar.graphics import style
from sugar.graphics.alert import ConfirmationAlert


class ControlPanel(hippo.CanvasBox):
    
    _VERSION = 5
    def __init__(self, handler):
        self._handler = handler
        hippo.CanvasBox.__init__(self, spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL)

        self.subject_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL)
        self.append(self.subject_box)
        
        self.progress_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL)
        self.append(self.progress_box)
        
        self.answering_mode_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL)
        self.append(self.answering_mode_box)
        
        self.flashcard_slider_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL)
        self.append(self.flashcard_slider_box)
        
        self.navigation_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_HORIZONTAL, xalign=hippo.ALIGNMENT_CENTER)
        self.append(self.navigation_box)
        
        self.info_box = hippo.CanvasBox(spacing=4, padding=5,
                orientation=hippo.ORIENTATION_VERTICAL, xalign=hippo.ALIGNMENT_CENTER)
        self.append(self.info_box)

        
        header_font = style.FONT_BOLD.get_pango_desc()
        header_font.set_size(12000)
        
        
        subject_header = hippo.CanvasText(text='Subject', color=style.COLOR_WHITE.get_int(),
                          font_desc= header_font)
        
        self.subject_combobox = ComboBox()
        self.subject_combo_changed_id = self.subject_combobox.connect("changed", self._on_combo_select)
        self.subject_box.append(subject_header)
        self.subject_box.append(hippo.CanvasWidget(widget=self.subject_combobox))
        self.subject_combobox.append_item(1, "some subject")
       
        self.progress_adjustment = gtk.Adjustment(0.0, 0.0, 001.0, 0.1, 0.0, 0.0)
        self.progressbar = gtk.ProgressBar(self.progress_adjustment)
        self.progressbar.set_fraction(0.4)
        progressbar_header = hippo.CanvasText(text='Progress', color=style.COLOR_WHITE.get_int(),
                          font_desc= header_font)

        self.progress_box.append(progressbar_header)
        self.progress_box.append(hippo.CanvasWidget(widget=self.progressbar))
        
        answering_mode_header = hippo.CanvasText(text='How did you answer?', color=style.COLOR_WHITE.get_int(),
                          font_desc= header_font)
        self.answering_mode_box.append(answering_mode_header)
        
        

        self.no_problem_button = gtk.ToggleButton()
        no_problem_label = gtk.Label()
        no_problem_label.set_markup(_('<big><b>Without a problem</b></big>'))
        self.no_problem_button.add(no_problem_label)
        self.no_problem_button.connect('toggled',self._on_toggle_button_pressed)
        self.answering_mode_box.append(hippo.CanvasWidget(widget=self.no_problem_button, box_height=style.GRID_CELL_SIZE * 0.5))
        self.problem_button = gtk.ToggleButton()
        problem_label = gtk.Label()
        problem_label.set_markup(_('<big><b>Hardly</b></big>'))
        self.problem_button.add(problem_label)
        self.problem_button.connect('toggled',self._on_toggle_button_pressed)
        self.answering_mode_box.append(hippo.CanvasWidget(widget=self.problem_button, box_height=style.GRID_CELL_SIZE * 0.5))
        self.forgotten_button = gtk.ToggleButton()
        forgotten_label = gtk.Label()
        forgotten_label.set_markup(_('<big><b>Answer forgotten</b></big>'))
        self.forgotten_button.add(forgotten_label)
        self.forgotten_button.connect('toggled',self._on_toggle_button_pressed)
        self.answering_mode_box.append(hippo.CanvasWidget(widget=self.forgotten_button, box_height=style.GRID_CELL_SIZE * 0.5))
        self.dunno_button = gtk.ToggleButton()
        dunno_label = gtk.Label()
        dunno_label.set_markup(_('<big><b>Answer unknown</b></big>'))
        self.dunno_button.add(dunno_label)
        self.dunno_button.connect('toggled',self._on_toggle_button_pressed)
        self.answering_mode_box.append(hippo.CanvasWidget(widget=self.dunno_button, box_height=style.GRID_CELL_SIZE * 0.5))
        
        flashcard_slider_header = hippo.CanvasText(text='Flashcard', color=style.COLOR_WHITE.get_int(),
                          font_desc= header_font)
        self.flashcard_slider_box.append(flashcard_slider_header)

        self.flashcard_slider = gtk.HScale()
        self.flashcard_slider_box.append(hippo.CanvasWidget(widget=self.flashcard_slider))
        self.flashcard_slider.set_digits(0)
        
        #Navigation buttons
        button = gtk.Button("Back",gtk.STOCK_GO_BACK)
        button.connect("clicked",self._on_previous_flashcard_clicked_cb,"Previous flashcard")
        self.navigation_box.append(hippo.CanvasWidget(widget=button))
        tooltips = gtk.Tooltips()
        tooltips.set_tip(button, "Previous flashcard", tip_private=None)
        button = gtk.Button("Forward",gtk.STOCK_GO_FORWARD)
        
        button.connect("clicked",self._on_next_flashcard_clicked_cb, "Next question")
        self.navigation_box.append(hippo.CanvasWidget(widget=button))
        tooltips.set_tip(button, "Next question")
        
        self.info_box.remove_all()
        self.add_buddy_info()
        self.add_version_info()

    def add_version_info(self):
        textwidget = hippo.CanvasText(text=_('V0.0'+ str(self._VERSION)),
            font_desc=pango.FontDescription('Sans 16'),
            color=style.COLOR_WHITE.get_int(),
            xalign=hippo.ALIGNMENT_CENTER)
        self.info_box.append(textwidget)
        
    def add_buddy_info(self):
        buddy_color = self._handler.buddy.props.color
        if not buddy_color:
            buddy_color = "#000000,#ffffff"

        icon = CanvasIcon(
            icon_name='computer-xo',
            xo_color=XoColor(buddy_color))

        nick = self._handler.buddy.props.nick
        if not nick:
            nick = ""
        name = hippo.CanvasText(text=nick, color=style.COLOR_WHITE.get_int())

        buddy_box = hippo.CanvasBox(padding=5)
        buddy_box.append(icon)
        buddy_box.append(name)
        self.info_box.append(buddy_box)


    def disable_widgets(self):
        pass
    
    def enable_widgets(self):
        pass
    
    def _on_combo_select(self, widget, data=None):
#TODO: Change the flashcard to one of that subject. Play with action_id=rowid to make comparations faster
        #When we load flashcard and set active the combobox, this is what is executed, handler blocked
        
        active_item = widget.get_active_item()
        if active_item:
            ##logging.info('A._on_combo_select, there is an active item in the combobox:%s' % active_item[0])
            if self._handler.current_questionnaire.current_subject != active_item[0]:
                
                #In this ocassion, I haven't decided wether the result should be recorded or not
                #self.record_result()
                #here, widget is self.subject_combobox
                current_flashcard = self._handler.current_questionnaire.get_card_with_subject(widget.get_active_item()[0])
                if current_flashcard:
                    self._handler.current_questionnaire.current_answer = None
                    self._handler.current_questionnaire.current_question = None
                    self._handler.current_questionnaire.current_flashcard = current_flashcard
                    #logging.info('A._on_combo_select, flashcard id = %s' % self._handler.current_questionnaire.current_flashcard)
                    self._handler.load_flashcard()  
        else:
            #logging.info('A._on_combo_select, there is NOT an active item in the combobox')
            pass
    
    def _on_toggle_button_pressed(self, widget, data=None):

        if widget.get_active():
            if widget is self.no_problem_button:
                self._handler.answering_mode = self._handler._EASYLY_ANSWERED
                answering_mode = _('I answered without a problem')
            elif widget is self.problem_button:
                self._handler.answering_mode = self._handler._HARDLY_ANSWERED
                answering_mode = _('Hardly but I answered correctly')
            elif widget is self.forgotten_button:
                self._handler.answering_mode = self._handler._FORGOTTEN_ANSWER
                answering_mode = _('I knew the answer but I didn\'t recall it')
            else:
                self._handler.answering_mode = self._handler._DUNNO_ANSWER
                answering_mode = _('I didn\'t know the answer')
            if self._handler.current_questionnaire.current_question:
                alert = ConfirmationAlert()
                alert.props.title = _("Do you want to add this result?")
                alert.props.msg = '"' + answering_mode + '"'
                self._handler.add_alert(alert)
                self._handler.disable_widgets()
                alert.connect('response', self._alert_add_result_cb, widget)
                alert.show()

    def _on_next_flashcard_clicked_cb(self, widget, data=None):

        #logging.info('A._on_next_flashcard_clicked_cb current_flashcard = %s' % (self._handler.current_questionnaire.current_flashcard))
        #Write the result in the database, 'cause the user decided is answered
        self._handler.current_questionnaire.current_answer = None
        self._handler.current_questionnaire.current_question = None
        self._handler.current_questionnaire.get_next_card()
        self._handler.load_flashcard()
#TODO: Scale bar is not yet synchronized
        if self.scale_adjustment.value < len(self._handler.current_questionnaire.flashcard_dict) + 1:
            self.scale_adjustment.set_value(self.scale_adjustment.value + 1)
        
    def _on_previous_flashcard_clicked_cb(self, widget, data=None):
        self._handler.current_questionnaire.current_answer = None
        self._handler.current_questionnaire.current_question = None
        self._handler.current_questionnaire.get_previous_card()
        self._handler.load_flashcard()
        if self.scale_adjustment.value > 0:
            self.scale_adjustment.set_value(self.scale_adjustment.value - 1)
            
    def _alert_add_result_cb(self, alert, response_id, toggle_button):
        if response_id is gtk.RESPONSE_OK:
            self._handler.record_result()
        toggle_button.set_active(False)
        self._handler.remove_alert(alert)
        self._handler.enable_widgets()
