# Copyright (C) 2008, Urko Fernandez <tturktime@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gobject
import random
import time
import hippo
import logging
import itertools

from sugar.graphics import style
from datetime import datetime

class hint:
    _DEFAULT_COLOR_STROKE = '#000000'
    _DEFAULT_COLOR_FILL = '#888888'
    _FLASH_TIMER = 100
    
    def __init__(self, handler, answer_dict, roundbox):
        self._handler = handler
        self._roundbox = roundbox
        self.flash_attempts = 1
        self.step_answer = None
        
        self._font = style.FONT_NORMAL.get_pango_desc()
        self._font.set_size(20000)
        #Send a copy to be modified, or else it will change the global answer dictionary
        self.create_answer_cyclic_lists(answer_dict.copy())

    def create_answer_cyclic_lists(self, answer_dict):
        wrong_answer_list=[]
        #Dictionary cannot change size during iteration
        temp_dict = answer_dict.copy()
        for answerid in temp_dict:
            if self._handler.current_questionnaire.get_karma_from_answer(answerid) <= 0:
                wrong_answer_list.append(answer_dict.pop(answerid))
        self.answer_cyclic_list = itertools.cycle(answer_dict.values()) 
        self.wrong_answer_cyclic_list = itertools.cycle(wrong_answer_list)
    
    def write_on_roundbox(self, text, color_stroke, color_fill):

        self.hinted_answer = hippo.CanvasText(text=text,
                size_mode=hippo.CANVAS_SIZE_WRAP_WORD,
                color=style.COLOR_WHITE.get_int(),
                font_desc=self._font)
        self._roundbox.remove_all()
        self._roundbox.props.border_color = style.Color(color_stroke).get_int()
        self._roundbox.props.background_color = style.Color(color_fill).get_int()
        self._roundbox.append(self.hinted_answer)
        #gobject.timeout_add(10, self.timeout, True)
        #logging.info('H.write_on_roundbox, written word = %s' % text)
    
    def clean_roundbox(self):
        self._roundbox.remove_all()
        # Also: self.hinted_answer.destroy()
        self._roundbox.props.border_color = style.Color(self._DEFAULT_COLOR_STROKE).get_int()
        self._roundbox.props.background_color = style.Color(self._DEFAULT_COLOR_FILL).get_int()
  
    def flash(self):
        (answerid, answer) = self.initialize_next_answer()
        #if user keeps asking for hints, give her more time
        if self.flash_attempts % 5 == 0:
            self._FLASH_TIMER += 10

        answer_in_a_list = []
        answer_in_a_list.append(answer)
        gobject.timeout_add(self._FLASH_TIMER, self.splash_text, answer_in_a_list, priority=gobject.PRIORITY_HIGH)
        self.flash_attempts += 1
            
    def speed_read(self):

        self.hinted_answer = None
        answer = self.answer_cyclic_list.next()
        (answerid, answer) = self.initialize_next_answer()
        word_list = answer.split(' ')
        #logging.info('H.speed_read, word_list = %s' % word_list)

        if len(word_list) == 1:
            wpm = 600 # word per minute
            delay_time = int(60.0 / wpm * 1000.0)
            gobject.timeout_add(delay_time, self.splash_text, list(answer), priority=gobject.PRIORITY_HIGH)
                #self.splash_text(char, flash_timer, color_stroke, color_fill, clean_all = False)
        else:
            wpm = 400 # word per minute
            delay_time = int(60.0 / wpm * 1000.0)            
            #logging.info('H.speed_read, inside for, adding current word_list is = %s' % word_list)
            gobject.timeout_add(delay_time, self.splash_text, word_list, priority=gobject.PRIORITY_HIGH)
    
        
    def splash_text(self, list):
        if len(list) == 0:
            self.clean_roundbox()
            #logging.info ('H.splash_text list is %s' % list )
            return False

        self.write_on_roundbox(list.pop(0), self.color_stroke, self.color_fill)
        return True

    def scramble(self):

#TODO: don't shuffle words, but constantly shuffle letters.
        (answerid, answer) = self.initialize_next_answer()
        for punctuation_mark in '!?.,:;-+':
            answer = answer.replace(punctuation_mark, ' ')
        
        answer_list = answer.split(' ')
        ##answer = answer.replace('  ')
        #Another way to remove those spaces
        try:
            while True:
                answer_list.remove('')
        except ValueError:
            pass
        
        scramble_mode = random.randint(0,2)
        scramble_mode = 0
        #logging.info('H.scramble scrambling using mode %s' % scramble_mode)
        answer = ''
        if not scramble_mode:
            #Mode 0 all sentence's words scrambled and each word too
            random.shuffle(answer_list)
        for word in answer_list:
            #Mode 1 words scrambled but they mantain position, mode 2, as in mode 1 but first and last letters not moved
            word=list(word)
            if scramble_mode == 2:
                scrambled_word = self.scramble_middle_word(word)
                #logging.info('H.scramble type of scramble_word=%s in mode 2' % (type(scrambled_word)))
            else:
                random.shuffle(word)
                #logging.info('H.scramble in mode 0/1 word is = %s' % word)
                #scrambled_word = str(word).strip('[]').replace("', ", '').replace("u'",'')[0:-1] Not working when u is first or last letter
        
                scrambled_word = str(word).replace(", u'",'').replace("'",'')[2:-1]
                #logging.info('H.scramble in mode 0/1 scramble_word is = %s' % scrambled_word)
            #logging.info('H.scramble type of answer=%s of scramble_word=%s' % (type(answer), type(scrambled_word)))
            answer += scrambled_word +' '
        
        answer = answer.lower()
        answer = answer.capitalize()
        answer = answer[0:-1]    
        self.write_on_roundbox(answer, self.color_stroke, self.color_fill)
    
    
    def scramble_middle_word(self, word_list):
        scrambled_text = word_list[1:-1]
        if len(scrambled_text) > 2:
            random.shuffle(scrambled_text)
            word_list[1:-1] = scrambled_text
        elif len(scrambled_text) == 2:
            (scrambled_text[0], scrambled_text[1]) = (scrambled_text[1], scrambled_text[0])
            word_list[1:-1] = scrambled_text
        return str(word_list).strip('[]').replace("', ", '').replace("u'",'')[0:-1]
         
        
    def step(self):
        if not self.step_answer:
            (answerid, answer) = self.initialize_next_answer()
            self.step_color_stroke = self.color_stroke
            self.step_color_fill = self.color_fill
            self.step_iter = iter(answer)
            self.step_answer = ''.join(self.step_iter.next())
            #logging.info('H.step first step, answer=%s step_answer=%s' % (answer, self.step_answer))
        else:
            try:
                next_char = self.step_iter.next()
            except StopIteration:
                self.step_answer = None
                #logging.info('H.step, finishing iteration')
            #logging.info('H.step some step, before join, step_answer=%s iter.next %s' % (self.step_answer, next_char))
            self.step_answer += next_char
            #logging.info('H.step some step, after join, step_answer=%s iter.next %s' % (self.step_answer, next_char))
        self.write_on_roundbox(self.step_answer, self.step_color_stroke, self.step_color_fill)
        
    def wrong_answer(self):
        (answerid, answer) = self.initialize_next_answer(wrong=True)
        self.write_on_roundbox(answer, self.color_stroke, self.color_fill)
                
    def initialize_next_answer(self, wrong=False):
        if wrong:
            answer = self.wrong_answer_cyclic_list.next()
        else:
            answer = self.answer_cyclic_list.next()
        answerid = self._handler.current_questionnaire.get_answerid(answer)
        buddyid = self._handler.current_questionnaire.get_answer_owner(answerid)
        color_stroke, color_fill = self._handler.current_questionnaire.get_buddy_colors(buddyid)
        self.color_stroke = color_stroke
        self.color_fill = color_fill
        return answerid, answer