# Copyright (C) 2008, Urko Fernandez <tturktime@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import logging
try:
    import sqlite3 as sqlite
    logging.info("Using built-in sqlite3")
except:
    logging.info("Using external pysqlite2")
    from pysqlite2 import dbapi2 as sqlite
import sys
import os
import Assimilate




class questionnaire:

    
    def __init__(self, name):
        
        logging.info('Q.__init__ opening %s' % name)
        self.database_name = name
        if not os.path.exists(self.database_name):
            os.chmod('data',0777)
            self.conn = sqlite.connect(self.database_name)
            
            self.curs = self.conn.cursor()
            self.create_questionnaire()
            #Initializing self.current_answer, question, flashcard and subject for later checking whether they're been assigned a value or not
        else:
            self.conn = sqlite.connect(self.database_name)
            self.curs = self.conn.cursor()
        self.current_answer = None
        self.current_question = None
        self.current_flashcard = None
        self.current_subject = None
      
    def create_questionnaire(self):
        
        self.create_tables()
        self.create_foreign_key_triggers()
    
    def create_tables(self):

        #SQlite treats data internally as strings, id string(TEXT) or integer(INT)?
        #rowid, every row has it's own unique id. It autoincrements and doesn't repeat unless you force it to.
        try:
            self.curs.execute('''
                CREATE TABLE subject (
                    id INTEGER NOT NULL PRIMARY KEY,
                    name TEXT
                )
            ''')
        except sqlite.OperationalError, data:
            print data
        
        try:
            self.curs.execute('''
                CREATE TABLE flashcard (
                    id INTEGER NOT NULL PRIMARY KEY,
                    deck INTEGER,
                    subjectid INTEGER
                        CONSTRAINT fk_subject_id REFERENCES subject(id)
                )
            ''')
        except sqlite.OperationalError, data:
            print data
            
        try:
            self.curs.execute('''
                CREATE TABLE tag (
                    id INTEGER NOT NULL PRIMARY KEY,
                    cardid INTEGER,
                    value TEXT,
                    UNIQUE (cardid, value)
                )
            ''')
        except sqlite.OperationalError, data:
            print data
        
#TODO: Maybe a unique big try block could suffice
        
        try:
            self.curs.execute('''
                CREATE TABLE related (
                    firstcard INTEGER NOT NULL
                        CONSTRAINT fk_flashcard_id REFERENCES flashcard(id) ON DELETE CASCADE,
                    secondcard INTEGER NOT NULL
                        CONSTRAINT fk_flashcard_id2 REFERENCES flashcard(id) ON DELETE CASCADE,
                    tagscore INTEGER,
                    answerscore INTEGER,
                    PRIMARY KEY (firstcard, secondcard)
                )
            ''')
                                  
        except sqlite.OperationalError, data:
            print data           
        
        try:
            self.curs.execute('''
                CREATE TABLE result (
                    id INTEGER NOT NULL PRIMARY KEY,
                    karma INTEGER,
                    mode INTEGER,
                    date REAL,
                    cardid INTEGER
                        CONSTRAINT fk_flashcard_id REFERENCES flashcard(id) ON DELETE CASCADE
                    )
            ''')
        except sqlite.OperationalError, data:
            print data  
        
        try:
            self.curs.execute('''
                CREATE TABLE buddy (
                    id INTEGER NOT NULL PRIMARY KEY,
                    nick TEXT,
                    colorstroke TEXT,
                    colorfill TEXT
                    )
                ''')
        except sqlite.OperationalError, data:
            print data
            
        try:
            self.curs.execute('''
            CREATE TABLE question (
                id INTEGER NOT NULL PRIMARY KEY,
                karma INTEGER,
                value TEXT,
                cardid INTEGER
                    CONSTRAINT fk_flashcard_id REFERENCES flashcard(id) ON DELETE CASCADE,
                buddyid INTEGER
                    CONSTRAINT fk_buddy_id REFERENCES buddy(id)
                )
            ''')
        except sqlite.OperationalError, data:
            print data  
            
        try:
            self.curs.execute('''
            CREATE TABLE answer (
                id INTEGER NOT NULL PRIMARY KEY,
                karma INTEGER,
                value TEXT,
                cardid INTEGER
                    CONSTRAINT fk_flashcard_id REFERENCES flashcard(id) ON DELETE CASCADE,
                buddyid INTEGER
                    CONSTRAINT fk_buddy_id REFERENCES buddy(id)
                )
            ''')
        except sqlite.OperationalError, data:
            print data
            
        try:
            self.curs.execute('''
            CREATE TABLE buddytag (
                tagid INTEGER NOT NULL
                    CONSTRAINT fk_tag_id REFERENCES tag(id),
                buddyid INTEGER NOT NULL
                    CONSTRAINT fk_buddy_id REFERENCES buddy(id),
                PRIMARY KEY (tagid, buddyid)
                )
            ''')
        except sqlite.OperationalError, data:
            print data   
            
        try:
            self.curs.execute('''
            CREATE TABLE votequestion (
                buddyid INTEGER NOT NULL
                    CONSTRAINT fk_buddy_id REFERENCES buddy(id),
                questionid INTEGER NOT NULL
                    CONSTRAINT fk_question_id REFERENCES question(id),
                buddykarma INTEGER NOT NULL,
                PRIMARY KEY (buddyid, questionid)
                )
            ''')
        except sqlite.OperationalError, data:
            print data   

        try:
            self.curs.execute('''
            CREATE TABLE voteanswer (
                buddyid INTEGER NOT NULL
                    CONSTRAINT fk_buddy_id REFERENCES buddy(id),
                answerid INTEGER NOT NULL
                    CONSTRAINT fk_answer_id REFERENCES answer(id),
                buddykarma INTEGER NOT NULL,
                PRIMARY KEY (buddyid, answerid)
                )
            ''')
        except sqlite.OperationalError, data:
            print data   
            
    def create_foreign_key_triggers(self):
        
        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_flashcard_subjectid_subject_id
                BEFORE INSERT ON [flashcard]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "flashcard" violates foreign key constraint "fki_flashcard_subjectid_subject_id"')
                  WHERE NEW.subjectid IS NOT NULL AND (SELECT id FROM subject WHERE id = NEW.subjectid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_flashcard_subjectid_subject_id
                BEFORE UPDATE ON [flashcard]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "flashcard" violates foreign key constraint "fku_flashcard_subjectid_subject_id"')
                    WHERE NEW.subjectid IS NOT NULL AND (SELECT id FROM subject WHERE id = NEW.subjectid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data
            
        # Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_flashcard_subjectid_subject_id
                BEFORE DELETE ON subject
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'delete on table "subject" violates foreign key constraint "fkd_flashcard_subjectid_subject_id"')
                  WHERE (SELECT subjectid FROM flashcard WHERE subjectid = OLD.id) IS NOT NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_related_firstcard_flashcard_id
                BEFORE INSERT ON [related]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "related" violates foreign key constraint "fki_related_firstcard_flashcard_id"')
                  WHERE (SELECT id FROM flashcard WHERE id = NEW.firstcard) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_related_firstcard_flashcard_id
                BEFORE UPDATE ON [related]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "related" violates foreign key constraint "fku_related_firstcard_flashcard_id"')
                      WHERE (SELECT id FROM flashcard WHERE id = NEW.firstcard) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data
        
        # Cascading Delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkdc_related_firstcard_flashcard_id
                BEFORE DELETE ON flashcard
                FOR EACH ROW BEGIN
                    DELETE FROM related WHERE related.firstcard = OLD.id;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_related_secondcard_flashcard_id
                BEFORE INSERT ON [related]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "related" violates foreign key constraint "fki_related_secondcard_flashcard_id"')
                  WHERE NEW.secondcard IS NOT NULL AND (SELECT id FROM flashcard WHERE id = NEW.secondcard) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_related_secondcard_flashcard_id
                BEFORE UPDATE ON [related]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "related" violates foreign key constraint "fku_related_secondcard_flashcard_id"')
                      WHERE NEW.secondcard IS NOT NULL AND (SELECT id FROM flashcard WHERE id = NEW.secondcard) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Cascading Delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkdc_related_secondcard_flashcard_id
                BEFORE DELETE ON flashcard
                FOR EACH ROW BEGIN
                    DELETE FROM related WHERE related.secondcard = OLD.id;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_result_cardid_flashcard_id
                BEFORE INSERT ON [result]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "result" violates foreign key constraint "fki_result_cardid_flashcard_id"')
                  WHERE NEW.cardid IS NOT NULL AND (SELECT id FROM flashcard WHERE id = NEW.cardid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_result_cardid_flashcard_id
                BEFORE UPDATE ON [result]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "result" violates foreign key constraint "fku_result_cardid_flashcard_id"')
                      WHERE NEW.cardid IS NOT NULL AND (SELECT id FROM flashcard WHERE id = NEW.cardid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Cascading Delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkdc_result_cardid_flashcard_id
                BEFORE DELETE ON flashcard
                FOR EACH ROW BEGIN
                    DELETE FROM result WHERE result.cardid = OLD.id;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data
            
        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_question_cardid_flashcard_id
                BEFORE INSERT ON [question]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "question" violates foreign key constraint "fki_question_cardid_flashcard_id"')
                  WHERE NEW.cardid IS NOT NULL AND (SELECT id FROM flashcard WHERE id = NEW.cardid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_question_cardid_flashcard_id
                BEFORE UPDATE ON [question]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "question" violates foreign key constraint "fku_question_cardid_flashcard_id"')
                      WHERE NEW.cardid IS NOT NULL AND (SELECT id FROM flashcard WHERE id = NEW.cardid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Cascading Delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkdc_question_cardid_flashcard_id
                BEFORE DELETE ON flashcard
                FOR EACH ROW BEGIN
                    DELETE FROM question WHERE question.cardid = OLD.id;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_question_buddyid_buddy_id
                BEFORE INSERT ON [question]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "question" violates foreign key constraint "fki_question_buddyid_buddy_id"')
                  WHERE NEW.buddyid IS NOT NULL AND (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_question_buddyid_buddy_id
                BEFORE UPDATE ON [question]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "question" violates foreign key constraint "fku_question_buddyid_buddy_id"')
                      WHERE NEW.buddyid IS NOT NULL AND (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_question_buddyid_buddy_id
                BEFORE DELETE ON buddy
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'delete on table "buddy" violates foreign key constraint "fkd_question_buddyid_buddy_id"')
                  WHERE (SELECT buddyid FROM question WHERE buddyid = OLD.id) IS NOT NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_answer_cardid_flashcard_id
                BEFORE INSERT ON [answer]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "answer" violates foreign key constraint "fki_answer_cardid_flashcard_id"')
                  WHERE NEW.cardid IS NOT NULL AND (SELECT id FROM flashcard WHERE id = NEW.cardid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_answer_cardid_flashcard_id
                BEFORE UPDATE ON [answer]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "answer" violates foreign key constraint "fku_answer_cardid_flashcard_id"')
                      WHERE NEW.cardid IS NOT NULL AND (SELECT id FROM flashcard WHERE id = NEW.cardid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Cascading Delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkdc_answer_cardid_flashcard_id
                BEFORE DELETE ON flashcard
                FOR EACH ROW BEGIN
                    DELETE FROM answer WHERE answer.cardid = OLD.id;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_answer_buddyid_buddy_id
                BEFORE INSERT ON [answer]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "answer" violates foreign key constraint "fki_answer_buddyid_buddy_id"')
                  WHERE NEW.buddyid IS NOT NULL AND (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
             ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_answer_buddyid_buddy_id
                BEFORE UPDATE ON [answer]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "answer" violates foreign key constraint "fku_answer_buddyid_buddy_id"')
                      WHERE NEW.buddyid IS NOT NULL AND (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
             ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_answer_buddyid_buddy_id
                BEFORE DELETE ON buddy
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'delete on table "buddy" violates foreign key constraint "fkd_answer_buddyid_buddy_id"')
                  WHERE (SELECT buddyid FROM answer WHERE buddyid = OLD.id) IS NOT NULL;
                END;
             ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_buddytag_tagid_tag_id
                BEFORE INSERT ON [buddytag]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "buddytag" violates foreign key constraint "fki_buddytag_tagid_tag_id"')
                  WHERE (SELECT id FROM tag WHERE id = NEW.tagid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_buddytag_tagid_tag_id
                BEFORE UPDATE ON [buddytag]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "buddytag" violates foreign key constraint "fku_buddytag_tagid_tag_id"')
                      WHERE (SELECT id FROM tag WHERE id = NEW.tagid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_buddytag_tagid_tag_id
                BEFORE DELETE ON tag
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'delete on table "tag" violates foreign key constraint "fkd_buddytag_tagid_tag_id"')
                  WHERE (SELECT tagid FROM buddytag WHERE tagid = OLD.id) IS NOT NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_buddytag_buddyid_buddy_id
                BEFORE INSERT ON [buddytag]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'insert on table "buddytag" violates foreign key constraint "fki_buddytag_buddyid_buddy_id"')
                  WHERE (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_buddytag_buddyid_buddy_id
                BEFORE UPDATE ON [buddytag]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ABORT, 'update on table "buddytag" violates foreign key constraint "fku_buddytag_buddyid_buddy_id"')
                      WHERE (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

        # Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_buddytag_buddyid_buddy_id
                BEFORE DELETE ON buddy
                FOR EACH ROW BEGIN
                  SELECT RAISE(ABORT, 'delete on table "buddy" violates foreign key constraint "fkd_buddytag_buddyid_buddy_id"')
                  WHERE (SELECT buddyid FROM buddytag WHERE buddyid = OLD.id) IS NOT NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data
            
    
# Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_votequestion_buddyid_buddy_id
                BEFORE INSERT ON [votequestion]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ROLLBACK, 'insert on table "votequestion" violates foreign key constraint "fki_votequestion_buddyid_buddy_id"')
                  WHERE (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_votequestion_buddyid_buddy_id
                BEFORE UPDATE ON [votequestion]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ROLLBACK, 'update on table "votequestion" violates foreign key constraint "fku_votequestion_buddyid_buddy_id"')
                      WHERE (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_votequestion_buddyid_buddy_id
                BEFORE DELETE ON buddy
                FOR EACH ROW BEGIN
                  SELECT RAISE(ROLLBACK, 'delete on table "buddy" violates foreign key constraint "fkd_votequestion_buddyid_buddy_id"')
                  WHERE (SELECT buddyid FROM votequestion WHERE buddyid = OLD.id) IS NOT NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_votequestion_questionid_question_id
                BEFORE INSERT ON [votequestion]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ROLLBACK, 'insert on table "votequestion" violates foreign key constraint "fki_votequestion_questionid_question_id"')
                  WHERE (SELECT id FROM question WHERE id = NEW.questionid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_votequestion_questionid_question_id
                BEFORE UPDATE ON [votequestion]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ROLLBACK, 'update on table "votequestion" violates foreign key constraint "fku_votequestion_questionid_question_id"')
                      WHERE (SELECT id FROM question WHERE id = NEW.questionid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_votequestion_questionid_question_id
                BEFORE DELETE ON question
                FOR EACH ROW BEGIN
                  SELECT RAISE(ROLLBACK, 'delete on table "question" violates foreign key constraint "fkd_votequestion_questionid_question_id"')
                  WHERE (SELECT questionid FROM votequestion WHERE questionid = OLD.id) IS NOT NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_voteanswer_buddyid_buddy_id
                BEFORE INSERT ON [voteanswer]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ROLLBACK, 'insert on table "voteanswer" violates foreign key constraint "fki_voteanswer_buddyid_buddy_id"')
                  WHERE (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_voteanswer_buddyid_buddy_id
                BEFORE UPDATE ON [voteanswer]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ROLLBACK, 'update on table "voteanswer" violates foreign key constraint "fku_voteanswer_buddyid_buddy_id"')
                      WHERE (SELECT id FROM buddy WHERE id = NEW.buddyid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_voteanswer_buddyid_buddy_id
                BEFORE DELETE ON buddy
                FOR EACH ROW BEGIN
                  SELECT RAISE(ROLLBACK, 'delete on table "buddy" violates foreign key constraint "fkd_voteanswer_buddyid_buddy_id"')
                  WHERE (SELECT buddyid FROM voteanswer WHERE buddyid = OLD.id) IS NOT NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign Key Preventing insert
        try:
            self.curs.execute('''
                CREATE TRIGGER fki_voteanswer_questionid_answer_id
                BEFORE INSERT ON [voteanswer]
                FOR EACH ROW BEGIN
                  SELECT RAISE(ROLLBACK, 'insert on table "voteanswer" violates foreign key constraint "fki_voteanswer_questionid_answer_id"')
                  WHERE (SELECT id FROM answer WHERE id = NEW.questionid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign key preventing update
        try:
            self.curs.execute('''
                CREATE TRIGGER fku_voteanswer_questionid_answer_id
                BEFORE UPDATE ON [voteanswer]
                FOR EACH ROW BEGIN
                    SELECT RAISE(ROLLBACK, 'update on table "voteanswer" violates foreign key constraint "fku_voteanswer_questionid_answer_id"')
                      WHERE (SELECT id FROM answer WHERE id = NEW.questionid) IS NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data

# Foreign key preventing delete
        try:
            self.curs.execute('''
                CREATE TRIGGER fkd_voteanswer_questionid_answer_id
                BEFORE DELETE ON answer
                FOR EACH ROW BEGIN
                  SELECT RAISE(ROLLBACK, 'delete on table "answer" violates foreign key constraint "fkd_voteanswer_questionid_answer_id"')
                  WHERE (SELECT questionid FROM voteanswer WHERE questionid = OLD.id) IS NOT NULL;
                END;
            ''')
        except sqlite.OperationalError, data:
            print data
            
    def first_rowid(self, table, cardid=None):
        if not cardid:
            self.curs.execute('SELECT min(rowid) FROM %s' % table)
        else:
            self.curs.execute('SELECT min(rowid) FROM %s WHERE cardid=%s' % (table, cardid))
        return self.curs.next()[0]
    
    def last_rowid(self, table, cardid=None):
        if not cardid:
            self.curs.execute('SELECT max(rowid) FROM %s' % table)
        else:
            self.curs.execute('SELECT max(rowid) FROM %s WHERE cardid=%s' % (table, cardid))
        return self.curs.next()[0]
        
    def next_rowid(self, table, rowid, cardid=None):
#TODO:Whatch for cases where there's no any! It returns None, taken care in the caller.
        if not cardid:
            self.curs.execute('SELECT min(rowid) FROM %s WHERE rowid>%s' % (table, rowid))
        else:
            self.curs.execute('SELECT min(rowid) FROM %s WHERE rowid>%s AND cardid=%s' % (table, rowid, cardid))
        return self.curs.next()[0]
    
    def previous_rowid(self, table, rowid):
        
        self.curs.execute('SELECT max(rowid) FROM %s WHERE rowid<%s' % (table, rowid))
        return self.curs.next()[0]
    
    def add_flashcard(self, deck):
        
        self.curs.execute('INSERT INTO flashcard (deck) VALUES (?)', (deck,))
        self.current_flashcard = self.curs.lastrowid
        self.read_flashcards()
        self.current_answer = None
        self.current_question = None
        self.conn.commit()
    

    def del_current_flashcard(self):
#TODO: If needed, convert it to delete any given flashcard
        self.current_answer = None
        self.current_question = None
        deleted_flashcard = self.current_flashcard
        deleted_subject = self.current_subject
        self.get_next_card()

        #logging.info('Q.del_current_flashcard, current flashcard is = %s' % self.current_flashcard )
        
        # If it returns the same card, then this is the last one in the database
        if self.current_flashcard == deleted_flashcard :
            self.current_flashcard = None
        # If the card to be deleted was the first one, update it to the next one
        if deleted_flashcard == self.first_flashcard:
            self.first_flashcard = self.current_flashcard
        self.curs.execute('DELETE FROM flashcard WHERE id=%s' % deleted_flashcard)
        # If the card to be deleted was the last one:
        if deleted_flashcard == self.last_flashcard:
            self.last_flashcard = self.last_rowid('flashcard')
        
        # Deletion of the tag is restricted if other buddies have tagged it
        self.curs.execute('SELECT id FROM tag WHERE cardid="%s"' % deleted_flashcard)
        
        for tag in self.tag_dict:
            self.curs.execute('DELETE FROM buddytag WHERE tagid=%s' % tag)
        self.curs.execute('DELETE FROM tag WHERE cardid=%s' % deleted_flashcard)
        #logging.info('Q.del_current_flashcard, query:DELETE FROM related WHERE firstcard=%s OR secondcard=%s' % ((deleted_flashcard,)*2))
        #logging.info('Q.del_current_flashcard, deleted_subject=%s and subject_dict=%s' % (deleted_subject, self.subject_dict))

#TODO: Create a method with the following delete subject functionality and use it in a new button on the modify menutoolbar
        if deleted_subject:
            #If the flashcard has a subject, delete it, otherwise do nothing
            self.curs.execute('SELECT count(*) FROM flashcard WHERE subjectid=%s AND id <> %s' % (deleted_subject, deleted_flashcard))
            if not self.curs.fetchone()[0]:
                self.del_subject(deleted_subject)
        self.conn.commit()
        
    def add_question(self, karma, question, cardid, buddyid):
        if question not in self.question_dict.values():
            self.curs.execute('INSERT INTO question (karma, value, cardid, buddyid) VALUES(?, ?, ?, ?)', [karma, question, cardid, buddyid])
            logging.info("Inserting question: %s buddyid=%s" % (question, buddyid))
            self.current_question = self.curs.lastrowid
            self.conn.commit()
            self.read_questions()
            #We are assuming the last question inserted doesn't have to be last_question
            #In case this is the first question added in a flashcard
            self.first_question = self.first_rowid('question', self.current_flashcard)
            self.last_question = self.last_rowid('question', self.current_flashcard)
    
    def del_question(self, questionid):
        #Whatch how you need a one element tuple as argument
        self.curs.execute('DELETE FROM question WHERE id=?', (questionid,))
        self.conn.commit()
        if questionid == self.first_question:
            self.first_question = self.first_rowid('question', self.current_flashcard)
        elif questionid == self.last_question:
            self.last_question = self.last_rowid('question', self.current_flashcard)
            
    def add_answer(self, karma, answer, cardid, buddyid):
        self.curs.execute('INSERT INTO answer (karma, value, cardid, buddyid) VALUES(?, ?, ?, ?)', [karma, answer, cardid, buddyid])
        self.current_answer = self.curs.lastrowid
        self.conn.commit()
        self.read_answers()
        #logging.info('Q.add_answer, for checking if commit is necesary prior to asking lastroiwd (%s)' % self.current_answer)
#TODO: is the last inserted rowid always bigger than what's in the database?
        self.first_answer = self.first_rowid('answer', self.current_flashcard)
        self.last_answer = self.last_rowid('answer', self.current_flashcard)
        self.related_cards()
        
                          
    def del_answer(self, answerid):
        self.curs.execute('DELETE FROM answer WHERE id=?', (answerid,))
        #We need to calculate again relation between this card and the rest
        self.curs.execute('DELETE FROM related WHERE firstcard=%s OR secondcard=%s' %(self.current_flashcard, self.current_flashcard))
        self.conn.commit()
        if answerid == self.first_answer:
            self.first_answer = self.first_rowid('answer', self.current_flashcard)
        elif answerid == self.last_answer:
            self.last_answer = self.last_rowid('answer', self.current_flashcard)
        self.related_cards()
        
    def change_subject(self, subject):
        #the first time it creates one, so first look if there's one with the same name
   
        self.curs.execute('SELECT id FROM subject WHERE name="%s"' % subject)
        subjectid = self.curs.fetchone()
        if not subjectid:
            self.curs.execute('INSERT INTO subject (name) VALUES (?)', (subject,))
            subjectid = self.curs.lastrowid
        else:
            subjectid = subjectid[0]
        #logging.info('Q.change_subject, cardid=%s and subjectid=%s' % (self.current_flashcard, subjectid))    
        self.curs.execute('UPDATE flashcard SET subjectid=%s WHERE id=%s' % (subjectid, self.current_flashcard))
        self.conn.commit()
    
    #def add_subject(self, subject):
            
    def del_subject(self, subjectid):
        self.curs.execute('DELETE FROM subject WHERE id=%s' % subjectid)
        #There is no referencial integrity, so be careful with inconsistencies

    def get_subject_from_card(self, id):
        self.curs.execute('SELECT subjectid FROM flashcard WHERE id=%s' % id)
        subject = self.curs.fetchone()
        if subject:
            #logging.info('Q.get_subject_from_card, subject=%s' % subject[0])
            return subject[0]
    
    def get_subject_name(self, id):
        self.curs.execute('SELECT name FROM subject WHERE id=%s' % id)
        subjectid = self.curs.fetchone()
        if subjectid:
            return subjectid[0]

    def get_card_with_subject(self, subjectid):
        self.curs.execute('SELECT id FROM flashcard WHERE subjectid="%s"' % subjectid)
        id = self.curs.fetchone()
        if id:
            #logging.info('Q.get_card_with_subject, subjectid: %s, first id is %s' % (subjectid, id[0]))
            return id[0]
    
    def read_subjects(self):
        self.curs.execute('SELECT id, name FROM subject')
        self.subject_dict = dict([(x[0], x[1]) for x in self.curs.fetchall()])
#TODO: it needs to point to the one associated to the card, done somewhere else?
        #self.current_subject = self.first_rowid('subject')

    def read_questions(self):

#TODO: check all database files, contrast with progress in an XML file and load 
        #If questions could jump from a questionnaire to other, then create more diferent connections
        #logging.info('Q.read_questions current_flashcard=%s' % self.current_flashcard)
        #When the DB is empty, because user deleted all the flashcards or started new db:
        if self.current_flashcard:
            self.curs.execute('SELECT id,value FROM question WHERE cardid=%s' % self.current_flashcard)
       
            #Creating a mapping with id and questions
            self.question_dict = dict([(x[0], x[1]) for x in self.curs.fetchall()])
            
            if not self.current_question:
                self.current_question = self.first_question = self.first_rowid('question', self.current_flashcard)
            self.last_question = self.last_rowid('question', self.current_flashcard)
            #logging.info ("current question in read_questions=%s, and cuestion_dict is %s" % (self.current_question, self.question_dict))        
        
    def read_answers(self, cardid=None):
        if not cardid:
            cardid = self.current_flashcard
            #This is for the first time opening an empty database
            if cardid:
                self.curs.execute('SELECT id, value FROM answer WHERE cardid=%s' % cardid)
                
                #Creating a mapping with id and answer
                self.answer_dict = dict([(x[0], x[1]) for x in self.curs.fetchall()])
                #logging.info ("q.read_answers, answer_dict=%s" % (self.answer_dict)) 
                if not self.current_answer:
                #attribute self.current_answer hasn't been initialized for the first time
                    self.current_answer = self.first_answer = self.first_rowid('answer', cardid)
                self.last_answer = self.last_rowid('answer', cardid)
                #logging.info ("current answer in read_answers=%s, and answer_dict is %s" % (self.current_answer, self.answer_dict))        
        else:
            self.curs.execute('SELECT id, value FROM answer WHERE cardid="%s"' % cardid)
            return dict([(x[0], x[1]) for x in self.curs.fetchall()])
        
        
        
            
        
    def next_question(self):
        #logging.info('Q.next_question, current_question is %s, first_question is %s and last %s' % (self.current_question, self.first_question, self.last_question))
        if self.current_question: 
            if self.current_question < self.last_question:
                self.current_question = self.next_rowid("question", self.current_question, self.current_flashcard)
            elif self.current_question:
                self.current_question = self.first_question
            return self.question_dict[self.current_question]  
#TODO: Now it will return None, but it needs to throw an exception
    
        
    def next_answer(self):
        #logging.info('Q.next_answer, current_answer is %s, first_answer is %s, last_answer is %s' % (self.current_answer, self.first_answer, self.last_answer))
        if self.current_answer:
            if self.current_answer < self.last_answer: 
                self.current_answer = self.next_rowid("answer", self.current_answer, self.current_flashcard)
            else:
                self.current_answer = self.first_answer
            return self.answer_dict[self.current_answer]
    
    def read_flashcards(self):
        self.curs.execute('SELECT id, deck FROM flashcard')
        #Creating a mapping with id
        self.flashcard_dict = dict([(x[0], x[1]) for x in self.curs.fetchall()])
        #logging.info ("flashcard_dict=%s" % (self.flashcard_dict)) 
        if not self.current_flashcard:
            self.current_flashcard = self.first_flashcard = self.first_rowid('flashcard')
            #logging.info('Q.read_flashcards, current_flashcard is %s' % self.current_flashcard)
        self.last_flashcard = self.last_rowid('flashcard')
    
    def get_next_card(self):
        self.current_answer = None

        if self.current_flashcard < self.last_flashcard: 
            self.current_flashcard = self.next_rowid("flashcard", self.current_flashcard)
            #logging.info('function get_next_card under questionnaire, value of current_flashcard = %s' % self.current_flashcard)
        else:
            self.current_flashcard = self.first_flashcard
        
        #no need to return any values, they're in the class instance
        
    def get_previous_card(self):
        self.current_answer = None
        #logging.info('Q.get_previous_card, last_flashcard=%s' % self.last_flashcard)
        if self.current_flashcard > self.first_flashcard:
            self.current_flashcard =self.previous_rowid('flashcard', self.current_flashcard)
            #logging.info('function get_previous_card under questionnaire, value of current_flashcard = %s' % self.current_flashcard)
        else:
            self.current_flashcard = self.last_flashcard
  
    def read_tags(self, cardid=None):

        if not cardid:
            cardid = self.current_flashcard
            self.curs.execute('SELECT id, value FROM tag WHERE cardid="%s"' % cardid)
            self.tag_dict = dict([(x[0], x[1]) for x in self.curs.fetchall()]) 
            #logging.info('Q.read_tags tag_dict=%s' % self.tag_dict)
        else:
            self.curs.execute('SELECT id, value FROM tag WHERE cardid="%s"' % cardid)
            return dict([(x[0], x[1]) for x in self.curs.fetchall()])
    
    def read_user_tags(self, buddyid, cardid=None):
        if not cardid:
            cardid = self.current_flashcard
        self.curs.execute('SELECT tagid, value FROM buddytag INNER JOIN tag ON tag.id=buddytag.tagid WHERE cardid=%s AND buddyid=%s'
                           % (cardid, buddyid))
        return dict([(x[0], x[1]) for x in self.curs.fetchall()])
        
    def get_number_of_tags(self, tagid):
        self.curs.execute('SELECT count(*) FROM buddytag WHERE tagid=%s' % tagid)
        return self.curs.fetchone()[0]
    
    def get_tagid(self, tag, cardid=None):
        if not cardid:
            cardid = self.current_flashcard
        self.curs.execute('SELECT id FROM tag WHERE value="%s" AND cardid=%s' % (tag, cardid))
        tag = self.curs.fetchone()
        if tag:
            return tag[0]
    
    def add_tag(self, cardid, tag):
        #logging.info('Q.add_tag: cardid is %s and tag is %s' % (cardid, tag))
        self.curs.execute('SELECT id FROM tag WHERE cardid="%s" AND value="%s"' % (cardid, tag))
        tagid = self.curs.fetchone()
        if not tagid:
            #logging.info('Q.add_tag, inserting %s %s' % (cardid, tag))
            self.curs.execute('INSERT INTO tag (cardid, value) VALUES (?, ?)', (cardid, tag))
            self.conn.commit()
            #logging.info ('Q.add_tag when inserting in lastrowid is %s' % self.curs.lastrowid)
            insertedTagId = self.curs.lastrowid
            #Creating a relation between current card and those tagged similarly
            self.related_cards()
        else:
            #we need to return this tagid
            #logging.info ('Q.add_tag in lastrowid when there\'s already one is %s' % tagid)
            insertedTagId= tagid[0]
        return insertedTagId
    
    def del_tag(self, tagid, buddyid):
        try:
            self.curs.execute('DELETE FROM buddytag WHERE tagid=%s AND buddyid=%s' % (tagid, buddyid))
            #When integrity fails, it ABORTs, so a commit is necessary before trying to delete tag
            self.conn.commit()
            try:
                self.curs.execute('DELETE FROM tag WHERE id=%s' % (tagid))
                self.conn.commit()
            except sqlite.IntegrityError, data:
                print data
        
        except sqlite.OperationalError, data:
            #Trying to delete a nonexisting tag
            print data

        logging.info('Q.del_tag, deleting tagid=%s for buddyid=%s' %  (tagid, buddyid))
        self.curs.execute('DELETE FROM related WHERE firstcard=%s OR secondcard=%s' %(self.current_flashcard, self.current_flashcard))
        #Update the relations between this newly modified card and the rest in the questionnaire
        self.related_cards()
        
        
    def add_buddy(self, buddy):
        #Check wether we have been in this database or not
        nick = buddy.props.nick
        #logging.info('Q.add_buddy, nick=%s' % (nick))
        color_stroke, color_fill = buddy.props.color.split(',')
        self.curs.execute('SELECT * FROM buddy WHERE nick="%s"' % nick)
        if not self.curs.fetchone():
            #logging.info('Q.add_buddy, stroke=%s, fill=%s' % (color_stroke, color_fill))
            self.curs.execute('INSERT INTO buddy (nick, colorstroke, colorfill) VALUES (?, ?, ?)' , (nick, color_stroke, color_fill))
        self.conn.commit()
            
    def add_buddy_tag(self, tagid, buddyid):
        #See if this buddy has submitted this same tag
        logging.info('Q.add_buddy_tag tagid=%s buddyid=%s' % (tagid, buddyid))
        self.curs.execute('SELECT * FROM buddytag WHERE tagid=%s AND buddyid=%s' % (tagid, buddyid))
        if not self.curs.fetchone():
            self.curs.execute('INSERT INTO buddytag (tagid, buddyid) VALUES (?, ?)', (tagid, buddyid))
            self.conn.commit()
            #logging.info('Q.add_buddy_tag Query:INSERT INTO buddytag VALUES (%s, %s)' %(tagid, buddyid))
            
    def get_buddyid(self, nick):   
        self.curs.execute('SELECT id FROM buddy WHERE nick="%s"' % nick)
        buddyid = self.curs.fetchone()
        if buddyid:
            return buddyid[0]
        
    def get_answerid(self, answer):
        self.curs.execute('SELECT id FROM answer WHERE value="%s"' % answer)
        answerid = self.curs.fetchone()
        if answerid:
            return answerid[0]
        
    def get_question_owner(self, questionid):
        self.curs.execute('SELECT buddyid FROM question WHERE id=%s' % questionid)
        return self.curs.fetchone()[0]
    
    def get_answer_owner(self, answerid):
        self.curs.execute('SELECT buddyid FROM answer WHERE id=%s' % answerid)
        return self.curs.fetchone()[0]
    
    def get_buddy_colors(self, buddyid):
        self.curs.execute('SELECT colorstroke, colorfill FROM buddy WHERE id=%s' % buddyid)
        return self.curs.fetchone()
    
    def add_result(self, karma, mode, date, cardid):
        #TODO: Calculate karma e.g if the user doesn't add an answer when there's none and says she did it OK
        self.curs.execute('INSERT INTO result (karma, mode, date, cardid) VALUES (?, ?, ?, ?)', (karma, mode, date, cardid))
        self.conn.commit()

    def related_cards(self):
        '''Creates or updates the relation between current card and others based on tag and answer similarity'''
#TODO: Measure performance of this method, if it's costly, executed only one of the above comparisions: tags or answers, but not both at the same time
        _MINIMUM_TAG_COUNT = 3
        flashcard_dict = self.flashcard_dict.copy()
        #We don't want to compare to our current flashcard:
        flashcard_dict.pop(self.current_flashcard)
        flashcard_dict.keys()
        #logging.info('Q.related_cards, beginning')
                
        #tag_dict is not updated until write_tag_list method executed
        current_tag_dict = self.read_tags(self.current_flashcard)
        
        for cardid in flashcard_dict:
            checked_tag_dict = self.read_tags(cardid)
            tag_count = 0
            checked_answer_dict = self.read_answers(cardid)
            answer_score = 0
            tag_score = 0
#TODO: both with tags and answers, if text is similar, puntuation could rise a bit            
            for our_tagid in current_tag_dict:
                for tagid in checked_tag_dict:
                    ratio = Assimilate.Assimilate.compare_sequences(current_tag_dict[our_tagid], checked_tag_dict[tagid])
                    if ratio >= 0.95:
                        tag_score = tag_score + min(self.get_number_of_tags(tagid), self.get_number_of_tags(our_tagid))
                        tag_count += 1
            for our_answerid in self.answer_dict:
                for answerid in checked_answer_dict:
                    ratio = Assimilate.Assimilate.compare_sequences(self.answer_dict[our_answerid], checked_answer_dict[answerid])
                    if ratio >= 0.95:
                        answer_score += 1
                        #logging.info('Q.related_cards,answers_dict=%s'%self.answer_dict)
                        #logging.info('Q.related_cards, answer_score=%s' % answer_score)
                        #logging.info('Q.related_cards, similar answers=%s and %s' % (self.answer_dict[our_answerid], checked_answer_dict[answerid]))
            if tag_count > 2 or answer_score > 0:
                #logging.info('Q.related_cards, my tag list=%s, current tag_list:%s' %(current_tag_dict, checked_tag_dict))
                #logging.info('Q.related_cards, equal tags=%s, difference between them=%s ' % (tag_count, tag_score))
                
                tag_score = tag_score + tag_count - _MINIMUM_TAG_COUNT
                self.add_relation(cardid, tag_score, answer_score)
            
       
    def add_relation(self, cardid, tagscore, answerscore):
        #Related current_flashcard to given, see first if the relation exists
        self.curs.execute('SELECT count(*) FROM related WHERE firstcard=(?) AND secondcard=(?) OR firstcard=(?) AND secondcard=(?)',(self.current_flashcard, cardid, cardid, self.current_flashcard))
        if not self.curs.fetchone()[0]:
            self.curs.execute('INSERT INTO related (firstcard, secondcard, tagscore, answerscore) VALUES (?, ?, ?, ?)', (self.current_flashcard, cardid,  tagscore, answerscore))
        else:
            #Only reaches this updating clause when cards are still related
            self.curs.execute('UPDATE related SET tagscore=(?), answerscore=(?) WHERE firstcard=(?) AND secondcard=(?)', ((tagscore, answerscore, self.current_flashcard, cardid)))
        self.conn.commit()
        
    def card_is_quizable(self, cardid=None):
        if not cardid:
            cardid = self.current_flashcard
        self.curs.execute('SELECT count(*) FROM question WHERE cardid=%s AND karma > 0' % cardid)
        return self.curs.fetchone()[0] > 3
    
    def get_karma_from_answer(self, answerid):
        self.curs.execute('SELECT karma FROM answer WHERE id=%s' % answerid)
        karma = self.curs.fetchone()
        if karma:
            return karma[0]
