# Copyright (C) 2008, Urko Fernandez <tturktime@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import logging
import os
import gtk
import itertools
import questionnaire

from gettext import gettext as _
from sugar.graphics.toolbutton import ToolButton
from sugar.activity import activity
from sugar.activity.activity import EditToolbar
from sugar.graphics.palette import Palette
from sugar.graphics.menuitem import MenuItem
from sugar.graphics.combobox import ComboBox
from sugar.graphics.alert import ConfirmationAlert, NotifyAlert


_QUESTION = 0
_ANSWER = 1
_SUBJECT = 2
_TAG = 3
_ADD_questionnaire = 4
_CHANGE_questionnaire = 5

_TOOLBAR_MODIFY = 10


class AssimilationEditToolbar(EditToolbar):

    def __init__(self, handler):
        EditToolbar.__init__(self)
        self._handler = handler
        
class QuizModeToolbar(gtk.Toolbar):
    
    def __init__(self, handler):
        gtk.Toolbar.__init__(self)
        self._handler = handler
        test = ToolButton()
        test.set_tooltip(_('Test mode'))
        #You need to set the label after setting tooltip, or else tooltip == label
        test.set_label('Test')
        test.connect('clicked', self.goto_test_panel)
        self.insert(test, -1)
        test.show()

        question = ToolButton()
        question.set_tooltip(_('Question mode'))
        question.set_label('Question')
        question.connect('clicked', self.goto_card_panel)
        self.insert(question, -1)
        question.show()
        contest = ToolButton()
        contest.set_tooltip(_('Contest mode'))
        contest.set_label('Contest')
        contest.connect('clicked', self.goto_contest_panel)
        self.insert(contest, -1)
        contest.show()
        
    def goto_test_panel(self, widget):
                #Some widgets are still visible
        self._handler.card_panel.answer_box.set_visible(False)
        self._handler.card_panel.get_answer_box.set_visible(False)
        self._handler.card_panel.answer_entry_box.set_visible(False)
        #self.answer_box.destroy()
        self._handler.card_panel.test_box.set_visible(True)
        self._hanlder.card_panel.show_choice_box.set_visible(True)
    
    def goto_card_panel(self, widget):
        self._handler.card_panel.answer_box.set_visible(True)
        self._handler.card_panel.get_answer_box.set_visible(True)
        self._handler.card_panel.answer_entry_box.set_visible(True)
        self._handler.card_panel.test_box.set_visible(False)
        self._hanlder.card_panel.show_choice_box.set_visible(False)
        
    
    def goto_contest_panel(self, widget):
        #self._handler.some_method_here
        pass


class ModifyToolbar(gtk.Toolbar):
    def __init__(self, handler):
        gtk.Toolbar.__init__(self)
        self._handler = handler
        
        add_question_palette = PaletteMenu(self._handler, _QUESTION)
        add_question_button = ToolButton()
        add_question_button.set_palette(add_question_palette)
        add_question_button.set_label('Add question')
        add_question_button.connect('clicked', self._on_add_question_button_clicked_cb)
        self.insert(add_question_button, -1)
        add_question_button.show()
        
        del_question_button = ToolButton()
        del_question_button.set_tooltip(_('Delete current question'))
        del_question_button.set_label('Del question')
        del_question_button.connect('clicked', self._on_del_question_button_clicked_cb)
        self.insert(del_question_button, -1)
        del_question_button.show()

        add_answer_palette = PaletteMenu(self._handler, _ANSWER)
        add_answer_button = ToolButton()
        add_answer_button.set_palette(add_answer_palette)
        #add_answer_button.set_tooltip(_('Add current answer'))
        add_answer_button.set_label('Add answer')
        add_answer_button.connect('clicked', self._on_add_answer_button_clicked_cb)
        #Signals: motion-notify-event,'enter-notify-event',selection-received, "state-changed",'focus','notify' this last one seems to work, but it's triggered also somewhere else
        self.insert(add_answer_button, -1)
        add_answer_button.show()

        del_answer_button = ToolButton()
        del_answer_button.set_tooltip(_('Delete current answer'))
        del_answer_button.set_label('Del answer')
        del_answer_button.connect('clicked', self._on_del_answer_button_clicked_cb)
        self.insert(del_answer_button, -1)
        del_answer_button.show()
        
        change_subject_palette = PaletteMenu(self._handler, _SUBJECT)
        change_subject_palette.connect('popup', self._on_change_subject_palette_popup_cb)
        change_subject_button = ToolButton()
        change_subject_button.set_palette(change_subject_palette)
        change_subject_button.set_label('Change subject')
        change_subject_button.connect('clicked', self._on_change_subject_button_clicked_cb)
        #When selected with keyboard, not needed at the moment
        #change_subject_button.connect('focus', self._on_change_subject_button_clicked_cb)
        self.insert(change_subject_button, -1)
        change_subject_button.show()
        
        del_tags_palette = PaletteMenu(self._handler, _TAG)
        del_tags_palette.connect('popup', self._on_del_tags_palette_popup_cb)
        del_tags_button = ToolButton()
        del_tags_button.set_palette(del_tags_palette)
        del_tags_button.set_label('Delete tags')
        del_tags_button.connect('clicked', self._on_del_tags_button_clicked_cb)
        self.insert(del_tags_button, -1)
        del_tags_button.show()

    def _on_add_question_button_clicked_cb(self, widget):
        widget.get_palette().popup(True)
#TODO: when clicked again, popdown
        
    def _on_add_answer_button_clicked_cb(self, widget, data=None):
        widget.get_palette().popup(True)
                
    def _on_del_question_button_clicked_cb(self, widget):
        if self._handler.current_questionnaire.current_question:
            alert = ConfirmationAlert()
            alert.props.title = "Are you sure you want to delete this question?"
            alert.props.msg = '"' + self._handler.card_panel.question_text.props.text + '"'
            self._handler.add_alert(alert)
            alert.connect('response', self._alert_delete_question_cb)
            alert.show()
          
    def _on_del_answer_button_clicked_cb(self, widget):
        if self._handler.current_questionnaire.current_answer:
            alert = ConfirmationAlert()
            alert.props.title = "Are you sure you want to delete this answer?"
            alert.props.msg = '"' + self._handler.card_panel.answer_text.props.text + '"'
            self._handler.add_alert(alert)
            alert.connect('response', self._alert_delete_answer_cb)
            alert.show()
        
    def _on_change_subject_button_clicked_cb(self, widget, data=None):
        widget.get_palette().popup(True)
#TODO: See how to notice pallete already popped up to pop it down
        logging.info('T._on_add_question_button_clicked_cb, notify_popup=%s' % widget.get_palette().palette_state)

    def _on_change_subject_palette_popup_cb(self, widget, data=None):
        self._handler.create_subject_combobox(widget.subject_combobox, widget.subject_combo_changed_id)
              
    def _on_delete_subject_button_clicked_cb(self, widget):
#TODO: if no other flashcard is associated to this topic, delete
        self._handler.create_subject_combobox(widget.get_palette().subject_combobox, widget.get_palette().subject_combo_changed_id)
        widget.get_palette().hbox.pack_start(widget.get_palette().subject_combobox)
        widget.get_palette().hbox.show_all()

        widget.get_palette().popup(True)
    
    def _on_del_tags_button_clicked_cb(self, widget):
        widget.get_palette().popup(True)
        
    def _on_del_tags_palette_popup_cb(self, widget):
        buddyid = self._handler.current_questionnaire.get_buddyid(self._handler.buddy.props.nick)
        tag_dict = self._handler.current_questionnaire.read_user_tags(buddyid)
        widget.tag_combobox.remove_all()
        for tagid in tag_dict:
            widget.tag_combobox.append_item(tagid, tag_dict[tagid])
        widget.tag_combobox.handler_block(widget.tag_combo_changed_id)
        widget.tag_combobox.set_active(0)
        widget.tag_combobox.handler_unblock(widget.tag_combo_changed_id)
        item = widget.tag_combobox.get_active_item()
        if item:
            widget.entry.set_text(item[1])

    def _alert_delete_answer_cb(self, alert, response_id):
        if response_id is gtk.RESPONSE_OK:
            next_answer = self._handler.current_questionnaire.next_rowid('answer', 
                self._handler.current_questionnaire.current_answer, 
                self._handler.current_questionnaire.current_flashcard)
            self._handler.current_questionnaire.del_answer(self._handler.current_questionnaire.current_answer)
            self._handler.current_questionnaire.current_answer = next_answer
            self._handler.current_questionnaire.read_answers()
            #logging.info('T._alert_delete_answer_cb, current_answer = %s and answer_dict = %s' % (self._handler.current_questionnaire.current_answer,self._handler.current_questionnaire.answer_dict))    
            if self._handler.current_questionnaire.current_answer:
                buddyid = self._handler.current_questionnaire.get_answer_owner(self._handler.current_questionnaire.current_answer)
                color_stroke, color_fill = self._handler.current_questionnaire.get_buddy_colors(buddyid)
                self._handler.card_panel.answer_rb.remove_all()
                self._handler.card_panel.write_answer(self._handler.current_questionnaire.answer_dict[self._handler.current_questionnaire.current_answer], color_stroke, color_fill)
            else:
                #If self._handler.write_answer(None), "no answers" message will be written
                self._handler.card_panel.write_answer('', self._handler._DEFAULT_COLOR_STROKE, self._handler._DEFAULT_COLOR_FILL)
        self._handler.remove_alert(alert)
        
    def _alert_delete_question_cb(self, alert, response_id):
        if response_id is gtk.RESPONSE_OK:
            current_question = self._handler.current_questionnaire.current_question
            self._handler.current_questionnaire.next_question()
            self._handler.current_questionnaire.del_question(current_question)
            self._handler.current_questionnaire.read_questions()
            if current_question != self._handler.current_questionnaire.current_question:
                buddyid = self._handler.current_questionnaire.get_question_owner(self._handler.current_questionnaire.current_question)
                color_stroke, color_fill = self._handler.current_questionnaire.get_buddy_colors(buddyid)
                self._handler.card_panel.question_rb.remove_all()
                self._handler.card_panel.write_question(self._handler.current_questionnaire.question_dict[self._handler.current_questionnaire.current_question], color_stroke, color_fill)
            else:
                #In this case, the deleted question was the last one, because they're both equal
                #logging.info('T._alert_delete_question_cb, deleting last question')
                self._handler.card_panel.write_question(None, self._handler._DEFAULT_COLOR_STROKE, self._handler._DEFAULT_COLOR_FILL)
                self._handler.current_questionnaire.current_question = None
                self._handler.current_questionnaire.first_question = None
                self._handler.current_questionnaire.last_question = None
        self._handler.remove_alert(alert)
        

               
class HelpToolbar(gtk.Toolbar):
    def __init__(self, handler):
        gtk.Toolbar.__init__(self)
        self._handler = handler

        flash = ToolButton()
        flash.set_tooltip(_('Give a flash hint'))
        flash.set_label(_('Flash'))
        flash.connect('clicked', self._on_show_flash_button_clicked_cb)
        self.insert(flash, -1)
        flash.show()
        
        # Speed reading
        speed_reading = ToolButton()
        speed_reading.set_tooltip(_('Speed read the answer'))
        speed_reading.set_label(_('Quick'))
        speed_reading.connect('clicked', self._on_speed_reading_button_clicked_cb)
        self.insert(speed_reading, -1)
        speed_reading.show()
        
        scramble = ToolButton()
        scramble.set_tooltip(_('Scrambles all the letters'))
        scramble.set_label(_('Scramble'))
        scramble.connect('clicked', self._on_scramble_button_clicked_cb)
        self.insert(scramble, -1)
        scramble.show()

        show_char = ToolButton()
        show_char.set_tooltip(_('Show one more character each step'))
        show_char.set_label(_('Step'))
        show_char.connect('clicked', self._on_show_char_button_clicked_cb)
        self.insert(show_char, -1)
        show_char.show()
        
        show_wrong = ToolButton()
        show_wrong.set_tooltip(_('Show a wrong answer'))
        show_wrong.set_label(_('Wrong'))
        show_wrong.connect('clicked', self._on_show_wrong_button_clicked_cb)
        self.insert(show_wrong, -1)
        show_wrong.show()
        
        help = ToolButton()
        help.set_tooltip(_('Ask for help'))
        help.set_label('Help')
        help.connect('clicked', self._on_ask_for_help_button_clicked_cb)
        self.insert(help, -1)
        help.show()

    def _on_show_flash_button_clicked_cb(self, widget):
        
        #logging.info('T._on_show_hint_button_clicked_cb before calling flash') 
        self._handler.answer_hint.flash()
        
    def _on_speed_reading_button_clicked_cb(self, widget):
        
        self._handler.answer_hint.speed_read()
    
    
    def _on_scramble_button_clicked_cb(self, widget):
        self._handler.answer_hint.scramble()
        
    def _on_show_char_button_clicked_cb(self, widget):
        self._handler.answer_hint.step()
    
    def _on_show_wrong_button_clicked_cb(self, widget):
        self._handler.answer_hint.wrong_answer()
              
    def _on_ask_for_help_button_clicked_cb(self, widget):
        pass
        
class questionnaireToolbar(gtk.Toolbar):
    def __init__(self, handler):
        gtk.Toolbar.__init__(self)
        self._handler = handler
        
        add_flashcard_button = ToolButton()
        add_flashcard_button.set_tooltip(_('Add a new flashcard to the questionnaire'))
        add_flashcard_button.set_label('Add flashcard')
        add_flashcard_button.connect('clicked', self._on_add_flashcard_button_clicked_cb)
        self.insert(add_flashcard_button, -1)
        add_flashcard_button.show()
        
        del_flashcard_button = ToolButton()
        del_flashcard_button.set_tooltip(_('Delete current flashcard from the questionnaire'))
        del_flashcard_button.set_label('Del flashcard')
        del_flashcard_button.connect('clicked', self._on_del_flashcard_button_clicked_cb)
        self.insert(del_flashcard_button, -1)
        del_flashcard_button.show()

        add_questionnaire_palette = PaletteMenu(self._handler, _ADD_questionnaire)
        add_questionnaire_button = ToolButton()
        add_questionnaire_button.set_palette(add_questionnaire_palette)
        add_questionnaire_button.set_label('Add questionnaire')
        add_questionnaire_button.connect('clicked', self._on_add_questionnaire_button_clicked_cb)
        self.insert(add_questionnaire_button, -1)
        add_questionnaire_button.show()
        

        del_questionnaire_button = ToolButton()
        del_questionnaire_button.set_tooltip(_('Delete current questionnaire?'))
        del_questionnaire_button.set_label('Del questionnaire')
        del_questionnaire_button.connect('clicked', self._on_del_questionnaire_button_clicked_cb)
        self.insert(del_questionnaire_button, -1)
        del_questionnaire_button.show()
        
        change_questionnaire_palette = PaletteMenu(self._handler, _CHANGE_questionnaire)
        change_questionnaire_button = ToolButton()
        change_questionnaire_button.set_palette(change_questionnaire_palette)
        change_questionnaire_button.set_label('Change questionnaire')
        change_questionnaire_button.connect('clicked', self._on_change_questionnaire_button_clicked_cb)
        change_questionnaire_palette.connect('popup', self._on_change_questionnaire_palette_popup_cb)
        self.insert(change_questionnaire_button, -1)
        change_questionnaire_button.show()
        
    def _on_add_flashcard_button_clicked_cb(self, widget):
        #When implementing Leitner system, use proper deck
        deck = 1
        self._handler.current_questionnaire.add_flashcard(deck)
        self._handler.load_flashcard()
        
    
    def _on_del_flashcard_button_clicked_cb(self, widget):

        #logging.info('T._on_del_flashcard_button_clicked_cb, current flashcard is %s' %self._handler.current_questionnaire.current_flashcard)
        #If there are no flashcards left and user tries to delete one
        if self._handler.current_questionnaire.current_flashcard:
            alert = ConfirmationAlert()
            alert.props.title = "Are you sure you want to delete current flashcard?"
            alert.props.msg = 'Flashcard number: ' + str(self._handler.current_questionnaire.current_flashcard)
            self._handler.add_alert(alert)
            alert.connect('response', self._alert_delete_flashcard_cb)
            alert.show()


    def _on_add_questionnaire_button_clicked_cb(self, widget):
        widget.get_palette().popup(True)

    def create_questionnaire_combobox(self, combobox, combo_changed_id):
        combobox.handler_block(combo_changed_id)
        combobox.remove_all()
        combobox.handler_unblock(combo_changed_id)
        i = itertools.count()
        for questionnaire_name in self._handler.questionnaire_dict:
            combobox.append_item(i.next(), questionnaire_name)
        combobox.handler_block(combo_changed_id)
        combobox.set_active(-1)
        combobox.handler_unblock(combo_changed_id)
        
    def _on_del_questionnaire_button_clicked_cb(self, widget):
        
        alert = ConfirmationAlert()
        alert.props.title = "Are you sure you want to delete current questionnaire?"
        alert.props.msg = 'questionnaire name: ' + self._handler.current_questionnaire.database_name.split('/')[1].split('.')[0]
        self._handler.add_alert(alert)
        alert.connect('response', self._alert_delete_questionnaire_cb)
        alert.show()
   
    def _on_change_questionnaire_button_clicked_cb(self, widget):
        widget.get_palette().popup(True)
    
    def _on_change_questionnaire_palette_popup_cb(self, widget):

        self.create_questionnaire_combobox(widget.change_questionnaire_combobox, widget.change_questionnaire_combo_changed_id)

    def _alert_delete_flashcard_cb(self, alert, response_id):
        if response_id is gtk.RESPONSE_OK:
            self._handler.current_questionnaire.del_current_flashcard()
            self._handler.load_flashcard()
        self._handler.remove_alert(alert) 
        
    def _alert_delete_questionnaire_cb(self, alert, response_id):
        if response_id is gtk.RESPONSE_OK:
            self._handler.questionnaire_dict.pop(self._handler.current_questionnaire.database_name.split('/')[1].split('.')[0])
            os.remove(self._handler.current_questionnaire.database_name)
            for questionnaire_name in self._handler.questionnaire_dict:
                self._handler.current_questionnaire =  self._handler.questionnaire_dict[questionnaire_name]
                break
            else:
                questionnaire_name = None
            self._handler.change_questionnaire(questionnaire_name)
        
            #If the user doesn't evaluate his answering, no result is to be recorded when flipping cards
            self.answering_mode = None
            
            #self._handler.load_flashcard()
        self._handler.remove_alert(alert)
                  
      
class PaletteMenu(Palette):

    def __init__(self, handler, menu_type):
        self._handler = handler
        self.hbox = gtk.HBox(False, 5)
        vbox = gtk.VBox(False, 5)
        
        if menu_type in (_QUESTION, _ANSWER, _SUBJECT, _TAG, _ADD_questionnaire):
            self.entry = gtk.Entry()
            vbox.pack_start(self.entry)
 
        if menu_type == _QUESTION:
            Palette.__init__(self, _('Add or rephrase a question with your own words'))
            question_button = gtk.Button(_('Add a new question'))
            question_button.connect('clicked', self._on_question_button_clicked_cb, self.entry)
            self.hbox.pack_start(question_button)
            self.entry.connect('activate', self._on_question_button_clicked_cb, self.entry)
            #Now answer takes it's value from the answer entry in the main canvas
        if menu_type == _ANSWER:
            Palette.__init__(self, _('Add or rephrase an answer with your own words'))
            answer_button = gtk.Button(_('Add a new answer'))
            answer_button.connect('clicked', self._on_answer_button_clicked_cb, self.entry)
            self.hbox.pack_start(answer_button)
            self.entry.connect('activate', self._on_answer_button_clicked_cb, self.entry)
        elif menu_type == _SUBJECT:
            Palette.__init__(self, _('Add or change the subject of this flashcard'))
            subject_button = gtk.Button(_('Change subject'))
            subject_button.connect('clicked', self._on_change_subject_button_clicked_cb, self.entry)
            self.entry.connect('activate', self._on_change_subject_button_clicked_cb, self.entry)
            self.hbox.pack_start(subject_button)
            self.subject_combobox =  ComboBox()
            self.subject_combobox.show()
            self.hbox.pack_start(self.subject_combobox)
            self.subject_combo_changed_id = self.subject_combobox.connect("changed", self._on_subject_combo_select)
        elif menu_type == _TAG:
            Palette.__init__(self, _('Select a tag to be removed'))
            tag_button = gtk.Button(_('Delete tag'))
            tag_button.connect('clicked', self._on_delete_tag_button_clicked_cb, self.entry)
            self.entry.connect('activate', self._on_delete_tag_button_clicked_cb, self.entry)
            self.hbox.pack_start(tag_button)
            self.tag_combobox =  ComboBox()
            self.tag_combobox.show()
            self.hbox.pack_start(self.tag_combobox)
            self.tag_combo_changed_id = self.tag_combobox.connect("changed", self._on_tag_combo_select)
        elif menu_type == _ADD_questionnaire:
            Palette.__init__(self, _('Name the new questionnaire'))
            add_questionnaire_button = gtk.Button(_('Add a new questionnaire'))
            add_questionnaire_button.connect('clicked', self._on_add_questionnaire_button_clicked_cb, self.entry)
            self.entry.connect('activate', self._on_add_questionnaire_button_clicked_cb, self.entry)
            self.hbox.pack_start(add_questionnaire_button)
        elif menu_type == _CHANGE_questionnaire:
            Palette.__init__(self, _('Select a questionnaire to switch to'))
            self.change_questionnaire_combobox =  ComboBox()
            self.change_questionnaire_combobox.show()
            self.hbox.pack_start(self.change_questionnaire_combobox)
            self.change_questionnaire_combo_changed_id = self.change_questionnaire_combobox.connect("changed", self._on_change_questionnaire_combo_select)
                
        self.hbox.show_all()
        vbox.pack_start(self.hbox)
        vbox.show_all()
        self.set_content(vbox)

    def _on_question_button_clicked_cb(self, button, question_entry):
        if question_entry.get_text():
            question = question_entry.get_text()
            (ratio, similar_question) = self._handler.find_similar_text(question, self._handler.current_questionnaire.question_dict)
                
            if ratio >= 0.75 and ratio < 0.95:
                alert = ConfirmationAlert()
                alert.props.title = "There is a similar question in this card:"
                alert.props.msg = '"' + similar_question + '"'
                self._handler.add_alert(alert)
                alert.connect('response', self._alert_question_cb, question)
                alert.show()
            elif ratio < 0.75:
                self._handler.add_question_to_current_card(question)
            
            self.popdown()
            question_entry.set_text('')
    
    def _on_answer_button_clicked_cb(self, button, answer_entry):
        if answer_entry.get_text():
            answer = answer_entry.get_text()
            (ratio, similar_answer) = self._handler.find_similar_text(answer, self._handler.current_questionnaire.answer_dict)
                
            if ratio >= 0.75 and ratio < 0.95:
                alert = ConfirmationAlert()
                alert.props.title = "There is a similar answer in this card:"
                alert.props.msg = '"' + similar_answer + '"'
                self._handler.add_alert(alert)
                alert.connect('response', self._alert_answer_cb, answer)
                alert.show()
            elif ratio < 0.75:
                self._handler.add_answer_to_current_card(answer)
                self._handler.card_panel.get_answer_button.set_sensitive(True)
                self._handler.card_panel.check_answer_button.set_sensitive(True)
            
            self.popdown()
            answer_entry.set_text('')
            
    def _on_change_subject_button_clicked_cb(self, button, subject_entry):
        if subject_entry.get_text():
            subject = subject_entry.get_text()
            (ratio, similar_subject) = self._handler.find_similar_text(subject, self._handler.current_questionnaire.question_dict)
            if ratio >= 0.75 and ratio < 0.95:
                alert = ConfirmationAlert()
                alert.props.title = "There is a similar subject in this card:"
                alert.props.msg = '"' + similar_subject + '"'
                self._handler.add_alert(alert)
                alert.connect('response', self._alert_subject_cb, answer)
                alert.show()
            elif ratio < 0.75:
                self._handler.add_subject_to_current_card(subject)
            
            self.popdown()
            subject_entry.set_text('')
        #logging.info('T._on_change_subject_button_clicked_cb current subject is = %s' %self._handler.current_questionnaire.current_subject )
     
    def _on_delete_tag_button_clicked_cb(self, button, tag_entry):
        tag = tag_entry.get_text()
        tagid = self._handler.current_questionnaire.get_tagid(tag)
        buddyid = self._handler.current_questionnaire.get_buddyid(self._handler.buddy.props.nick)
        logging.info('T._on_delete_tag_button_clicked_cb going to delete %s' % tagid)
        self._handler.current_questionnaire.del_tag(tagid, buddyid)
        if tag:
            tag_entry.set_text('')
        self._handler.write_tag_list()
        self.popdown()

    def _on_add_questionnaire_button_clicked_cb(self, button, questionnaire_entry):
        new_questionnaire = questionnaire_entry.get_text()
        if new_questionnaire:
            logging.info('T._on_add_questionnaire_button_clicked_cb questionnaire=%s' % questionnaire)
            if os.path.exists('data/' + new_questionnaire + '.sqlite'):
                alert = NotifyAlert()
                alert.props.title = "There is already a questionnaire with the name:"
                alert.props.msg = '"' + new_questionnaire + '"'
                self._handler.add_alert(alert)
                alert.connect('response', self._alert_add_questionnaire_cb)
                alert.show()
            else:
                self._handler.questionnaire_dict[new_questionnaire] = questionnaire.questionnaire('data/' + new_questionnaire + '.sqlite')
            logging.info('T._on_add_questionnaire_button_clicked_cb questionnaire_dict=%s'%self._handler.questionnaire_dict)
            if new_questionnaire:
                questionnaire_entry.set_text('')
        self.popdown()
    
    
    def _on_subject_combo_select(self, widget, data=None):
        #logging.info('T._on_combo_select combo active item %s' % self.subject_combobox.get_active_item())
        self.entry.set_text(self.subject_combobox.get_active_item()[1])

    def _on_tag_combo_select(self, widget, data=None):
        item = self.tag_combobox.get_active_item()
        if item:
            self.entry.set_text(item[1])
            
    def _on_change_questionnaire_combo_select(self, combobox, data=None):
        item = combobox.get_active_item()

        if item:
            logging.info('T._on_change_questionnaire_combo_select, active item=%s and %s' % (item[0],item[1]))
            self._handler.change_questionnaire(item[1])
        logging.info('T._on_change_questionnaire_combo_select going to popdown!')
        combobox.handler_block(self.change_questionnaire_combo_changed_id)
        combobox.set_active(-1)
        combobox.handler_unblock(self.change_questionnaire_combo_changed_id)
        #Popdown is not working 
        self.popup()
        
    def _alert_add_questionnaire_cb(self, alert, response_id):
        if response_id is gtk.RESPONSE_OK:
            self._handler.remove_alert(alert)       


    def _alert_answer_cb(self, alert, response_id, answer):
        if response_id is gtk.RESPONSE_OK:
            #logging.debug('T._alert_answer_cb Clicked Continue, answer=%s' % answer)
            self._handler.add_answer_to_current_card(answer)
            self._handler.card_panel.get_answer_button.set_sensitive(True)
            self._handler.card_panel.check_answer_button.set_sensitive(True)
        self._handler.remove_alert(alert)  
 
    def _alert_question_cb(self, alert, response_id, question):
        if response_id is gtk.RESPONSE_OK:
            #logging.debug('T._alert_question_cb Clicked Continue, question=%s' % question)
            self._handler.add_question_to_current_card(question)
        self._handler.remove_alert(alert)    

    def _alert_subject_cb(self, alert, response_id, subject):
        if response_id is gtk.RESPONSE_OK:
            #logging.debug('T._alert_question_cb Clicked Continue, question=%s' % question)
            self._handler.add_subject_to_current_card(question)
        self._handler.remove_alert(alert)    
